/*
 * gaim AOL Instant Messenger Client
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef USE_APPLET
#include <gnome.h>
#include <applet-widget.h>
#endif /* USE_APPLET */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <gtk/gtk.h>
#include "aim.h"
#include "pixmaps/admin_icon.xpm"
#include "pixmaps/aol_icon.xpm"
#include "pixmaps/free_icon.xpm"
#include "pixmaps/dt_icon.xpm"
#include "pixmaps/no_icon.xpm"
#include "pixmaps/login_icon.xpm"
#include "pixmaps/logout_icon.xpm"

#include "pixmaps/buddyadd.xpm"
#include "pixmaps/buddydel.xpm"
#include "pixmaps/buddychat.xpm"
#include "pixmaps/im.xpm"
#include "pixmaps/info.xpm"
#include "pixmaps/permadd.xpm"
#include "pixmaps/permdel.xpm"

static GtkTooltips *tips;
static GtkWidget *buddies;
static GtkWidget *editpane;
static GtkWidget *buddypane;
static GtkWidget *permitpane;
static GtkWidget *tree;
static GtkWidget *permtree;
static GtkWidget *imbutton, *infobutton, *chatbutton;
static GtkWidget *addbutton, *rembutton;
static GtkWidget *addpermbutton, *rempermbutton;


GtkWidget *awaymess;
GtkWidget *blist;
GtkWidget *bpmenu;
GtkWidget *awaymenu;
struct buddy_pounce *bproot = NULL;
struct category *croot=NULL;
char ourname[80];
int permdeny;
int button_pix;
GList *permit;
GList *deny;
struct conversation *cnvroot;
struct log_conversation *l;

void destroy_buddy()
{
	if (blist)
		gtk_widget_destroy(blist);
	blist=NULL;
}

void update_num_categories()
{
        struct category *c = croot;
        int pres, total;
        char buf[BUF_LONG];
        struct person *m;
        if (!show_cat_nums)
                return;
        while(c) {
                m = c->members;
                pres = 0;
                total = 0;
                while(m) {
                        if (m->present)
                                pres++;
                        total++;


                        m = m->next;
                }
                g_snprintf(buf, sizeof(buf), "%s  (%d/%d)", c->name, pres, total);
                gtk_label_set(GTK_LABEL(c->label), buf);
                c = c->next;

        }
        
}

void update_show_idlepix()
{
        struct category *c = croot;
        struct person *m;

        while (c) {
                m = c->members;
                while(m) {
                        if (showidle)
                                gtk_widget_show(m->idletime);
                        else
                                gtk_widget_hide(m->idletime);

                        if (showpix)
                                gtk_widget_show(m->pix);
                        else
                                gtk_widget_hide(m->pix);
                        m = m->next;
                }
                c = c->next;
        }
}

static GtkWidget *adjust_pic(GtkWidget *button, const char *c, gchar **xpm)
{
        GdkPixmap *pm;
        GdkBitmap *bm;
        GtkWidget *pic;
        GtkWidget *label;

        if (button_pix) {
		pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm,
						  NULL, (gchar **)xpm);
		pic = gtk_pixmap_new(pm, bm);
		gtk_widget_show(pic);
		label = GTK_BIN(button)->child;
		gtk_container_remove(GTK_CONTAINER(button), label);
		gtk_container_add(GTK_CONTAINER(button), pic);
        } else {
		label = gtk_label_new(c);
		gtk_widget_show(label);
		pic = GTK_BIN(button)->child;
		gtk_container_remove(GTK_CONTAINER(button), pic);
		gtk_container_add(GTK_CONTAINER(button), label);
	}


        gtk_container_resize_children(GTK_CONTAINER(blist));


        
}


void update_button_pix()
{

        adjust_pic(addbutton, "Add", (gchar **)buddyadd_xpm);
	adjust_pic(rembutton, "Remove", (gchar **)buddydel_xpm);
	adjust_pic(chatbutton, "Chat", (gchar **)buddychat_xpm);
        adjust_pic(imbutton, "IM", (gchar **)im_xpm);
        adjust_pic(infobutton, "Info", (gchar **)info_xpm);
        adjust_pic(addpermbutton, "Add", (gchar **)permadd_xpm);
        adjust_pic(rempermbutton, "Remove", (gchar **)permdel_xpm);


        
}


#ifdef USE_APPLET
void applet_destroy_buddy() {
        struct category *c, *c2;
        struct person *p, *p2;
        c = croot;
        while(c) {
                p = c->members;
                while(p) {
                        p2 = p;
                        p = p->next;
                        g_free(p2);
                }
                c2 = c;
                c = c->next;
                g_free(c2);
        }
        croot=NULL;
	destroy_buddy();
        applet_widget_unregister_callback(APPLET_WIDGET(applet),"away");
	if(!awaymess)
        	applet_widget_register_callback(APPLET_WIDGET(applet),
      			"away",
                	_("Away Message"),
                	show_away_mess,
                	NULL);
	else
		applet_widget_register_callback(APPLET_WIDGET(applet),
                	"away",
                        _("Back"),
                        (AppletCallbackFunc) do_im_back,
                        NULL);
	if(!blist) {
		applet_widget_register_callback(APPLET_WIDGET(applet),
			"buddy",
			_("Buddy List"),
			(AppletCallbackFunc)make_buddy,
			NULL);
	}
}
#endif


void signoff()
{
	struct category *c, *c2;
	struct person *p, *p2;
	c = croot;
	while(c) {
		p = c->members;
		while(p) {
			p2 = p;
			p = p->next;
			g_free(p2);
		}
		c2 = c;
		c = c->next;
		g_free(c2);
	}
        croot=NULL;
	toc_close();
        destroy_all_dialogs();
        destroy_buddy();
	hide_progress("");
#ifdef USE_APPLET
        applet_widget_unregister_callback(APPLET_WIDGET(applet),"signoff");
	applet_widget_unregister_callback(APPLET_WIDGET(applet),"away");
	applet_widget_unregister_callback(APPLET_WIDGET(applet),"buddy");
        applet_widget_register_callback(APPLET_WIDGET(applet),
                "signon",
                _("Signon"),
                applet_show_login,
                NULL);
#else
        display_login();
#endif /* USE_APPLET */
}

void handle_click_category(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	if (event->type == GDK_2BUTTON_PRESS) {
		if (GTK_TREE_ITEM(widget)->expanded)
			gtk_tree_item_collapse(GTK_TREE_ITEM(widget));
		else
			gtk_tree_item_expand(GTK_TREE_ITEM(widget));
	} else {
	}
}

void handle_click_person(GtkWidget *widget, GdkEventButton *event, struct person *p)
{

        if (event->type == GDK_2BUTTON_PRESS) {
                struct conversation *c;

                c = find_conversation(p->name);

                if (c != NULL) {
                        gdk_window_raise(c->window->window);
                } else {
                        c = new_conversation(p->name);
                }
        } else {
                // Anything for other buttons? :)
	}
}


int is_buddy(char *cname)
{
	struct category *cat;
	struct person *mem;

	cat = croot;

	while (cat) {
		mem = cat -> members;
		cat = cat->next;
		while (mem) {
			if (!strcasecmp(mem->name, cname)) {
				return 1;
			}
			mem = mem->next;
		}
	}
	return 0;
}

void remove_person(char *cat, char *person)
{
	struct category *c;
	int elsewhere=0;
	struct person *p, *p2;

	
	c = croot;
	while(c) {
		if (!strcasecmp(c->name, cat)) {
			p = c->members;
			p2 = c->members;
			if (!p) continue;
			
			while(p) {
				if (!strcasecmp(p->name, person)) {

//					printf("Found '%s' in group '%s'\n", p
//					       ->name, c->name);
					if (p == c->members)
                                                c->members = p->next;
					else
						p2->next = p->next;
					gtk_tree_remove_item(GTK_TREE(c->tree), p->item);
					g_free(p);
				}
				p2 = p;
				p=p->next;
			}
		} else {
			p = c->members;
			while(p) {
				if (!strcasecmp(p->name, person))
					elsewhere++;
				p=p->next;
			}
		}
		c=c->next;
	}
	if (!elsewhere) {
		char buf[BUF_LONG];
		g_snprintf(buf, sizeof(buf)/2, "toc_remove_buddy %s", condense(person));
#if 0
		fprintf(stdout, "Sending '%s'\n",buf);
#endif
		sflap_send(buf, -1, TYPE_DATA);
        }
        update_num_categories();
}

void remove_category(char *cat)
{
        struct category *c = croot, *c2 = croot;
        struct person *p, *p2;


	
        while(c) {
                if (!strcasecmp(c->name, cat)) {
                        // hide it first for speed
                        gtk_widget_hide(c->item);
                        p = c->members;
                        while(p) {
                                p2 = p->next;
                                remove_person(c->name, p->name);
                                // use this so it gets taken care of
                                // the right way, tho maybe a little
                                // slower;
                                p = p2;
                        }
                        gtk_tree_remove_item(GTK_TREE(buddies), c->item);
                        if (c == croot) {
                                if (c->next)
                                        croot = c->next;
                                else
                                        croot = NULL;
                        } else {
                                c2->next = c->next;
                        }

                        g_free(c);




                        c = croot;
                        while (c) {
//                                printf("A%s\n", c->name);
                                c = c->next;
                        }
                        return;
                        
                

                }
                c2 = c;
                c = c->next;
        }
}


void build_config(char *s, int len)
{
	struct category *c = croot;
        struct person *p;
        GList *plist = permit;
        GList *dlist = deny;
        int pos=0;
        if (!permdeny)
                permdeny = 1;
	pos += g_snprintf(&s[pos], len - pos, "toc_set_config {m %d\n", permdeny);
	while(c) {
		pos += g_snprintf(&s[pos], len - pos, "g %s\n", c->name);
		p = c->members;
		while(p) {
			pos += g_snprintf(&s[pos], len - pos, "b %s\n", condense(p->name));
			p=p->next;
		}
		c=c->next;
        }
        while(plist) {
                pos += g_snprintf(&s[pos], len - pos, "p %s\n", condense((char *)plist->data));
                plist=plist->next;

        }
        while(dlist) {
                pos += g_snprintf(&s[pos], len - pos, "d %s\n", condense((char *)dlist->data));
                dlist=dlist->next;
        }
	pos += g_snprintf(&s[pos], len - pos, "}");
}

void build_permit_tree()
{
	GtkWidget *ti;
        GtkWidget *sub;
        GList *plist = permit;
        GList *dlist = deny;

        gtk_tree_clear_items(GTK_TREE(permtree), 0, -1);

        ti = gtk_tree_item_new_with_label("Permit");
        sub = gtk_tree_new();
        gtk_widget_show(ti);
        gtk_widget_show(sub);
        gtk_tree_prepend(GTK_TREE(permtree), ti);
        gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), sub);
        gtk_tree_item_expand(GTK_TREE_ITEM(ti));
        
        while(plist) {
                ti = gtk_tree_item_new_with_label((char *)plist->data);
                gtk_widget_show(ti);
                gtk_tree_prepend(GTK_TREE(sub), ti);
                plist = plist->next;
        }


        ti = gtk_tree_item_new_with_label("Deny");
        sub = gtk_tree_new();
        gtk_widget_show(ti);
        gtk_widget_show(sub);
        gtk_tree_prepend(GTK_TREE(permtree), ti);
        gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), sub);
        gtk_tree_item_expand(GTK_TREE_ITEM(ti));
        
        while(dlist) {
                ti = gtk_tree_item_new_with_label((char *)dlist->data);
                gtk_widget_show(ti);
                gtk_tree_prepend(GTK_TREE(sub), ti);
                dlist = dlist->next;
        }

	
}

void build_edit_tree()
{
	GtkWidget *ti;
	GtkWidget *sub;
	struct category *c;
	struct person *p;
	struct log_conversation *l;

        gtk_tree_clear_items(GTK_TREE(tree), 0, -1);
        
	gtk_widget_hide(tree);
	
	c = croot;
	while(c) {
		ti = gtk_tree_item_new_with_label(c->name);
		sub = gtk_tree_new();
		gtk_widget_show(ti);
		gtk_widget_show(sub);
		gtk_tree_prepend(GTK_TREE(tree), ti);
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), sub);
		gtk_tree_item_expand(GTK_TREE_ITEM(ti));
                gtk_object_set_user_data(GTK_OBJECT(ti), c->name);
                p = c->members;
		while(p) {
			ti = gtk_tree_item_new_with_label(p->name);
			gtk_widget_show(ti);
			gtk_tree_prepend(GTK_TREE(sub), ti);
                        gtk_object_set_user_data(GTK_OBJECT(ti), c->name);
                        p=p->next;
                        
 		}
		c=c->next;
	}

	gtk_widget_show(tree);
	
}

int add_person(char *category, char *person)
{
	struct category *c;
        struct person *p;
        char *who = g_malloc(64);
	int result=0;


        strcpy(who, condense(person));

	c = croot;
	
        while(c) {
                p = c->members;
		while(p) {
			if (!strcasecmp(who, condense(p->name))) {
				g_free(who);
				return 1;
			}
                        p = p->next;
                }
                c = c->next;
        }

        g_free(who);
	
        c = croot;
	while(c) {
		if (!strcasecmp(category, c->name)) {
			GdkPixmap *pm;
			GdkBitmap *bm;
			GtkWidget *box;
			p = (struct person *)g_new(struct person, 1);
			if (!p)
				return 0;
			p->next = c->members;
			p->present=0;
			c->members = p;
			p->item = gtk_tree_item_new();

			box = gtk_hbox_new(FALSE, 1);
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, 
				NULL, (gchar **)login_icon_xpm);
			p->pix = gtk_pixmap_new(pm, bm);
			
			gtk_widget_show(p->pix);

			p->label = gtk_label_new(person);
			gtk_misc_set_alignment(GTK_MISC(p->label), 0.0, 0.5);

			p->idletime = gtk_label_new("");
		
			gtk_box_pack_start(GTK_BOX(box), p->pix, FALSE, FALSE, 1);
			gtk_box_pack_start(GTK_BOX(box), p->label, TRUE, TRUE, 1);
			gtk_box_pack_start(GTK_BOX(box), p->idletime, FALSE, FALSE, 1);
	

			gtk_container_add(GTK_CONTAINER(p->item), box);
		
			gtk_widget_show(p->label);
			gtk_widget_show(box);

			gtk_object_set_user_data(GTK_OBJECT(p->item), p);
		

			gtk_signal_connect(GTK_OBJECT(p->item), 
			   "button_press_event",
			   GTK_SIGNAL_FUNC(handle_click_person),
			   p);
			g_snprintf(p->name, sizeof(p->name), "%s", person);
			gtk_tree_append(GTK_TREE(c->tree),p->item);
			result++;
		}
		c=c->next;
        }
	return result;
}


void add_category(char *category)
{
	struct category *c;
	c = (struct category *)g_new(struct category, 1);
	if (!c)
		return;
	
	strncpy(c->name, category, sizeof(c->name));
        c->item = gtk_tree_item_new();
        c->label = gtk_label_new(c->name);
        gtk_misc_set_alignment(GTK_MISC(c->label), 0.0, 0.5);
        gtk_widget_show(c->label);
        gtk_container_add(GTK_CONTAINER(c->item), c->label);
	c->tree = gtk_tree_new();
	gtk_widget_show(c->item);
	gtk_widget_show(c->tree);
	gtk_tree_append(GTK_TREE(buddies), c->item);
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(c->item), c->tree);
	gtk_tree_item_expand(GTK_TREE_ITEM(c->item));
	gtk_signal_connect(GTK_OBJECT(c->item), 
      			   "button_press_event",
			   GTK_SIGNAL_FUNC(handle_click_category),
                           NULL);
        gtk_object_set_user_data(GTK_OBJECT(c->item), NULL);
	c->next = croot;
	c->members = NULL;
        croot = c;
}


static void do_del_buddy(GtkWidget *w, GtkTree *tree)
{
	GtkLabel *label, *plabel;
	GtkWidget *item, *pitem;
	char *c, *d;
	GList *i;
	int level;
	char buf[BUF_LONG];
	
	i = GTK_TREE_SELECTION(tree);
	if (i) {
		item = GTK_WIDGET(i->data);
		label = GTK_LABEL(GTK_BIN(item)->child);
		gtk_label_get(label, &c);
		level = GTK_TREE(item->parent)->level;
		if (level > 0) {
			pitem = GTK_WIDGET(GTK_TREE(item->parent)->tree_owner);
			plabel = GTK_LABEL(GTK_BIN(pitem)->child);
			gtk_label_get(plabel, &d);
//			fprintf(stdout,"Delete buddy '%s' from '%s'\n", c, d);
			remove_person(d, c);
                } else {
                        remove_category(c);
//			fprintf(stdout,"Delete group '%s'\n",c);
		}
		gtk_tree_clear_items(GTK_TREE(tree), 0, -1);
		build_edit_tree();
		build_config(buf, sizeof(buf)/2);
		sflap_send(buf, -1, TYPE_DATA);
        } else {
//                fprintf(stdout,"Delete empty buddy!\n");
        }
        update_num_categories();
}

static void do_del_perm(GtkWidget *w, GtkTree *permtree)
{
	GtkLabel *label, *plabel;
	GtkWidget *item, *pitem;
	char *c, *d;
	GList *i;
	
        GList *plist;
        GList *dlist;
	int level;
	char buf[BUF_LONG];

        plist = permit;
        dlist = deny;
        
	i = GTK_TREE_SELECTION(permtree);
	if (i) {
		item = GTK_WIDGET(i->data);
		label = GTK_LABEL(GTK_BIN(item)->child);
		gtk_label_get(label, &c);
		level = GTK_TREE(item->parent)->level;
		if (level > 0) {
			pitem = GTK_WIDGET(GTK_TREE(item->parent)->tree_owner);
			plabel = GTK_LABEL(GTK_BIN(pitem)->child);
			gtk_label_get(plabel, &d);
  //      		fprintf(stdout,"Delete buddy '%s' from '%s'\n", c, d);
                        if (!strcasecmp(d, "Permit")) {
                                while(plist) {
                                        if (!strcasecmp((char *)(plist->data), c)) {
                                                permit = g_list_remove(permit, plist->data);
                                                break;
                                        }

                                        plist = plist->next;
                                }

                        } else {
                                while(dlist) {
                                        if (!strcasecmp((char *)(dlist->data), c)) {
                                                deny = g_list_remove(deny, dlist->data);
                                                
                                                break;
                                        }
                                        dlist = dlist->next;
                                }

                        }

                        
                } else {
                        /* Can't delete groups here! :) */
                        return;
		}
		gtk_tree_clear_items(GTK_TREE(permtree), 0, -1);
		build_permit_tree();
		build_config(buf, sizeof(buf)/2);
		sflap_send(buf, -1, TYPE_DATA);
	}
//		fprintf(stdout,"Delete empty buddy!\n");
}




void import_callback(GtkWidget *widget, void *null)
{
        show_import_dialog();
}

void export_callback(GtkWidget *widget, void *null)
{
        show_export_dialog();
}



void do_quit()
{
	exit(0);
}

void add_buddy_callback(GtkWidget *widget, void *dummy)
{
        GList *i;
        char *cat = NULL;
        i = GTK_TREE_SELECTION(tree);
        if (i)
                cat = gtk_object_get_user_data(GTK_OBJECT(i->data));
        show_add_buddy(NULL, cat);
}

void add_perm_callback(GtkWidget *widget, void *dummy)
{
        show_add_perm(NULL);
}


void info_callback(GtkWidget *widget, GtkTree *tree)
{
	GList *i;
	char send[256];
	struct person *p;
	i = GTK_TREE_SELECTION(tree);
	if (i) {
		p = gtk_object_get_user_data(GTK_OBJECT(i->data));
	}
	g_snprintf(send, sizeof(send), "toc_get_info %s", p->name);
	sflap_send(send, -1, TYPE_DATA);
	
}


void chat_callback(GtkWidget *widget, GtkTree *tree)
{
	join_chat();
}

struct category *find_category(char *who)
{
	struct category *c;
	struct person *m;
	char *theirname = g_malloc(strlen(who)+1);

	strcpy(theirname, condense(who));
	c = croot;
	while (c) {
		m = c->members;
		while (m) {
			 if (!strcasecmp(condense(m->name), theirname)) {
				g_free(theirname);
				return c;
			}
			m = m->next;
		}
		c = c->next;
	}

	g_free(theirname);
	return NULL;	
	
}
struct person *get_person(char *who)
{
	struct category *c;
        struct person *p;
        char *lookinfor = g_malloc(strlen(who)+1);

        strcpy(lookinfor, condense(who));
        c = croot;
        while(c) {
                p=c->members;
//                printf("Looking for %s in cat %s\n", lookinfor, c->name);
		while(p) {
			if (!strcasecmp(condense(p->name),lookinfor)) {
				g_free(lookinfor);
				return p;
			}
			p=p->next;
		}
		c=c->next;
	}
	g_free(lookinfor);
	return NULL;
}


void rem_bp(GtkWidget *w, struct buddy_pounce *b)
{
	struct buddy_pounce *c = bproot, *d = bproot;

	while(c) {

		if (c == b) {
			
			d->next = c->next;
			
			if (c == bproot) {
				bproot = c->next;


			}

			g_free(c);

			do_bp_menu();
			return;

		}

		d = c;
                c = c->next;
	}


}

void do_pounce(char *name)
{
        char *who = g_malloc(64);
        char *buf = g_malloc(BUF_LONG);
        
        struct buddy_pounce *b, *x;
        struct conversation *c;
        
        strcpy(who, condense(name));

        b = bproot;

        while(b) {
                x = b->next;
                if (!strcasecmp(who, condense(b->name))) {
                        c = find_conversation(name);
                        if (c == NULL)
                                c = new_conversation(name);

                        write_to_conv(c->text, ourname, b->message, WFLAG_SEND);

                        escape_text(b->message);
                        
                        g_snprintf(buf, BUF_LONG/2, "toc_send_im %s \"%s\"", name, b->message);

                        sflap_send(buf, -1, TYPE_DATA);
                        
                        rem_bp(NULL, b);
                        
                }
                b = x;
        }
        g_free(who);

}

void new_bp_callback(GtkWidget *w, void *dummy)
{
        show_new_bp();
}



void do_bp_menu()
{
	GtkWidget *menuitem, *mess, *messmenu;
	static GtkWidget *remmenu;
        GtkWidget *remitem;
        GtkWidget *sep;
	GList *l;
	struct buddy_pounce *b = bproot;
	

	l = gtk_container_children(GTK_CONTAINER(bpmenu));
	
	while(l) {
		gtk_widget_destroy(GTK_WIDGET(l->data));
		l = l->next;
	}

	remmenu = gtk_menu_new();
	
	menuitem = gtk_menu_item_new_with_label("New Buddy Pounce");
	gtk_menu_append(GTK_MENU(bpmenu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(new_bp_callback), NULL);


	while(b) {

		remitem = gtk_menu_item_new_with_label(b->name);
		gtk_menu_append(GTK_MENU(remmenu), remitem);
		gtk_widget_show(remitem);
		gtk_signal_connect(GTK_OBJECT(remitem), "activate", GTK_SIGNAL_FUNC(rem_bp), b);

		b = b->next;

	}
	
	menuitem = gtk_menu_item_new_with_label("Remove Buddy Pounce");
	gtk_menu_append(GTK_MENU(bpmenu), menuitem);
	gtk_widget_show(menuitem);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), remmenu);
        gtk_widget_show(remmenu);


        sep = gtk_hseparator_new();
	menuitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(bpmenu), menuitem);
	gtk_container_add(GTK_CONTAINER(menuitem), sep);
	gtk_widget_set_sensitive(menuitem, FALSE);
	gtk_widget_show(menuitem);
	gtk_widget_show(sep);

	b = bproot;
	
	while(b) {

		menuitem = gtk_menu_item_new_with_label(b->name);
		gtk_menu_append(GTK_MENU(bpmenu), menuitem);
                messmenu = gtk_menu_new();
                gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), messmenu);
                gtk_widget_show(menuitem);
                


                mess = gtk_menu_item_new_with_label(b->message);
                gtk_menu_append(GTK_MENU(messmenu), mess);
                gtk_widget_show(mess);

                b = b->next;

	}

}


gint log_timeout(char *name)
{
	struct person *p = get_person(name);

	if(!p)
		return FALSE;
			
	if (!p->present)
		gtk_widget_hide(p->item);
	else
		set_buddy(p);
	return FALSE;
}

void set_buddy(struct person *p)
{
	char *who;
	char infotip[256];
        char idlet[16], sep[4];
        char hours[256], minutes[256], idle[256], warn[256];
	int i, hrs, min, tm, t;
	GdkPixmap *pm;
	GdkBitmap *bm;
	
	if (p->present) {
                g_snprintf(idlet, sizeof(idlet), "(%d)", p->idle);
		gtk_widget_hide(p->idletime);
		if (p->idle)
			gtk_label_set(GTK_LABEL(p->idletime), idlet);
		else
			gtk_label_set(GTK_LABEL(p->idletime), "");
                if (showidle)
                        gtk_widget_show(p->idletime);


                time((time_t *)&t);
                tm = (t - p->signon) /  60;
                hrs = tm / 60;
                min = tm % 60;

                if (min) {
			if (min == 1)
				g_snprintf(minutes, sizeof(minutes), "%d minute.", min);
			else
				g_snprintf(minutes, sizeof(minutes), "%d minutes.", min);
                        sprintf(sep, ", ");
		} else {
			if (!hrs)
				g_snprintf(minutes, sizeof(minutes), "%d minutes.", min);
			else {
				minutes[0] = '.';
				minutes[1] = '\0';
			}
		}

		if (hrs) {
                        if (hrs == 1)
				g_snprintf(hours, sizeof(hours), "%d hour%s", hrs, sep);
			else
                                g_snprintf(hours, sizeof(hours), "%d hours%s", hrs, sep);
		} else
			hours[0] = '\0';
		
		if (p->idle) {
                        if (p->idle == 1)
				g_snprintf(idle, sizeof(idle), "Idle: %d minute.\n", p->idle);
			else
				g_snprintf(idle, sizeof(idle), "Idle: %d minutes.\n", p->idle);
		} else
			idle[0] = '\0';

		if (p->evil) {
			g_snprintf(warn, sizeof(warn), "Warnings: %d%%\n", p->evil);

		} else
			warn[0] = '\0';
		
		i = g_snprintf(infotip, sizeof(infotip), "Name: %s                \nLogged in: %s%s\n%s%s", p->name, hours, minutes, warn, idle);
		
		gtk_tooltips_set_tip(tips, GTK_WIDGET(p->item), infotip, "");
		

		if (!GTK_WIDGET_VISIBLE(p->item)) {
			play_sound(BUDDY_ARRIVE);

			who = g_malloc(sizeof(p->name) + 10);
			strcpy(who, p->name);
	
			gtk_label_set(GTK_LABEL(p->label), who);
			g_free(who);
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm,
				NULL, (gchar **)login_icon_xpm);
			gtk_widget_hide(p->pix);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
                        if (showpix)
                                gtk_widget_show(p->pix);
			gtk_widget_show(p->item);
                        gtk_timeout_add(10000, (GtkFunction) log_timeout, p->name);
                        update_num_categories();
                        update_show_idlepix();
			return;
		}
		gtk_widget_hide(p->pix);
		if (p->uc & UC_AOL) {

			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, 
				NULL, (gchar **)aol_icon_xpm);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
		} else if (p->uc & UC_NORMAL) {
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, 
				NULL, (gchar **)free_icon_xpm);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
		} else if (p->uc & UC_ADMIN) {
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, 
				NULL, (gchar **)admin_icon_xpm);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
		} else if (p->uc & UC_UNCONFIRMED) {
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, 
				NULL, (gchar **)dt_icon_xpm);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
		} else {
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm,
				NULL, (gchar **)no_icon_xpm);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
                }
                if (showpix)
                        gtk_widget_show(p->pix);
		
	


	} else {
		if (GTK_WIDGET_VISIBLE(p->item)) {
			play_sound(BUDDY_LEAVE);
			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm,
				NULL, (gchar **)logout_icon_xpm);
			gtk_widget_hide(p->pix);
			gtk_pixmap_set(GTK_PIXMAP(p->pix), pm, bm);
                        if (showpix)
                                gtk_widget_show(p->pix);
                        gtk_timeout_add(10000, (GtkFunction)log_timeout, p->name);
                        update_num_categories();
                        update_show_idlepix();

		}
	}
}

void parse_buddy_list(char *config, int init)
{
	char *c;
        char current[256];
        char *name;
        char buf[BUF_LONG];
        char buf2[BUF_LONG];
        char bufperm[BUF_LONG];
        char bufdeny[BUF_LONG];
        int pos=0, ppos=0, dpos=0;
        /* Clean out the permit/deny list!*/
        g_list_free(permit);
        g_list_free(deny);
        permit = NULL;
        deny = NULL;

        
        pos+=g_snprintf(buf, sizeof(buf)/2, "toc_add_buddy");
        ppos+=g_snprintf(bufperm, sizeof(bufperm)/2, "toc_add_permit");
        dpos+=g_snprintf(bufdeny, sizeof(bufdeny)/2, "toc_add_deny");
        /* Skip first token, "CONFIG:xxx" */
        c = strtok(config + sizeof(struct sflap_hdr), "\n");
        sscanf(c + strlen(c) - 1, "%d", &permdeny);
        if (permdeny == 0)
                permdeny = 1;
        while((c=strtok(NULL,"\n"))) {
		if (*c == 'g') {
			strncpy(current,c+2, sizeof(current));
			add_category(current);
		} else if (*c == 'b') {
			pos+=g_snprintf(buf + pos, sizeof(buf)/2 -pos, " %s", condense(c + 2));
			add_person(current, c+2);
                } else if (*c == 'p') {
                        ppos+=g_snprintf(bufperm + ppos, sizeof(bufperm)/2 -ppos, " %s", condense(c + 2));
                        name = g_malloc(strlen(c+2) + 2);
                        g_snprintf(name, strlen(c+2) + 1, "%s", c+2);
                        permit = g_list_prepend(permit, name);
                } else if (*c == 'd') {
                        dpos+=g_snprintf(bufdeny + dpos, sizeof(bufdeny)/2 -dpos, " %s", condense(c + 2));
                        name = g_malloc(strlen(c+2) + 2);
                        g_snprintf(name, strlen(c+2) + 1, "%s", c+2);
                        deny = g_list_prepend(deny, name);
                }
	}
#if 0
	fprintf(stdout, "Sending message '%s'\n",buf);
#endif
	sflap_send(buf, -1, TYPE_DATA);
        sflap_send(bufperm, -1, TYPE_DATA);
        sflap_send(bufdeny, -1, TYPE_DATA);
        if (init) {
                g_snprintf(buf2, sizeof(buf2), "toc_init_done");
                sflap_send(buf2, -1, TYPE_DATA);
        }

}



void show_buddy_list(char *config)
{
	/* Build the buddy list, based on *config */

	GtkWidget *sw;
	GtkWidget *menu;
	GtkWidget *menubar;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *sep;
	GtkWidget *menuitem;
        GtkWidget *notebook;
        GtkWidget *label;
	GtkWidget *bbox;
        GtkWidget *tbox;
        GtkWidget *xbox;
        GtkWidget *pbox;
        
	blist      = gtk_window_new(GTK_WINDOW_TOPLEVEL);


	/* Icon */
	gtk_widget_realize(blist);
	aol_icon(blist->window);
        
	menubar = gtk_menu_bar_new();
	
	menu = gtk_menu_new();

	menuitem = gtk_menu_item_new_with_label("File");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
	gtk_widget_show(menuitem);


	menuitem = gtk_menu_item_new_with_label("Add A Buddy");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(add_buddy_callback), NULL);


	sep = gtk_hseparator_new();
	menuitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_container_add(GTK_CONTAINER(menuitem), sep);
	gtk_widget_set_sensitive(menuitem, FALSE);
	gtk_widget_show(menuitem);
	gtk_widget_show(sep);

        menuitem = gtk_menu_item_new_with_label("Import Buddy List");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(import_callback), NULL);


        menuitem = gtk_menu_item_new_with_label("Export Buddy List");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(export_callback), NULL);

	

	sep = gtk_hseparator_new();
	menuitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_container_add(GTK_CONTAINER(menuitem), sep);
	gtk_widget_set_sensitive(menuitem, FALSE);
	gtk_widget_show(menuitem);
	gtk_widget_show(sep);

	menuitem = gtk_menu_item_new_with_label("Signoff");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(signoff), NULL);


#ifndef USE_APPLET
	menuitem = gtk_menu_item_new_with_label("Quit");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(do_quit), NULL); 
#else
	menuitem = gtk_menu_item_new_with_label("Close");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(applet_destroy_buddy), NULL);
#endif

	menu = gtk_menu_new();

	menuitem = gtk_menu_item_new_with_label("Tools");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
	gtk_widget_show(menuitem);

	menuitem = gtk_menu_item_new_with_label("Preferences");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_prefs), NULL);

	awaymenu = gtk_menu_new();

	menuitem = gtk_menu_item_new_with_label("Away");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), awaymenu);
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);

        do_away_menu();

        bpmenu = gtk_menu_new();
        
        menuitem = gtk_menu_item_new_with_label("Buddy Pounce");
        gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), bpmenu);
        gtk_menu_append(GTK_MENU(menu), menuitem);
        gtk_widget_show(menuitem);

        do_bp_menu();

	menu = gtk_menu_new();

	menuitem = gtk_menu_item_new_with_label("Help");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), menu);
	gtk_menu_item_right_justify(GTK_MENU_ITEM(menuitem));
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
	gtk_widget_show(menuitem);

	menuitem = gtk_menu_item_new_with_label("About");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_about), NULL);

	gtk_widget_show(menubar);


	vbox       = gtk_vbox_new(FALSE, 10);
        
        notebook = gtk_notebook_new();

 
        

        /* Do buddy list stuff */

        buddypane = gtk_vbox_new(FALSE, 0);
        
        imbutton   = gtk_button_new_with_label("Im");
	infobutton = gtk_button_new_with_label("Info");
	chatbutton = gtk_button_new_with_label("Chat");

	hbox       = gtk_hbox_new(TRUE, 10);

	buddies    = gtk_tree_new();
	sw         = gtk_scrolled_window_new(NULL, NULL);
	



        
	tips = gtk_tooltips_new();
	gtk_object_set_data(GTK_OBJECT(blist), "Buddy List", tips);
	
 
	
	/* Now the buddy list */
#if (GTK_MINOR_VERSION > 1) || ((GTK_MICRO_VERSION > 4) && (GTK_MINOR_VERSION > 0))
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), buddies);
#else
	gtk_container_add(GTK_CONTAINER(sw),buddies);
#endif
#if 0
	/* we don't do this for any of the other panes, lets keep it
	   homogenous.
	*/
	gtk_container_border_width(GTK_CONTAINER(sw), 10);	 
#endif
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				       GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(sw,200,200);
	gtk_widget_show(buddies);
	gtk_widget_show(sw);

	/* Put the buttons in the hbox */
	gtk_widget_show(imbutton);
	gtk_widget_show(chatbutton);
	gtk_widget_show(infobutton);

	gtk_box_pack_start(GTK_BOX(hbox), imbutton, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), infobutton, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), chatbutton, TRUE, TRUE, 0);
        gtk_container_border_width(GTK_CONTAINER(hbox), 10);


	gtk_tooltips_set_tip(tips,infobutton, "Information on selected Buddy", "Penguin");
	gtk_tooltips_set_tip(tips,imbutton, "Send Instant Message", "Penguin");
	gtk_tooltips_set_tip(tips,chatbutton, "Start/join a Buddy Chat", "Penguin");

        gtk_box_pack_start(GTK_BOX(buddypane), sw, TRUE, TRUE, 0);
        gtk_box_pack_start(GTK_BOX(buddypane), hbox, FALSE, FALSE, 0);

        gtk_widget_show(hbox);
        gtk_widget_show(buddypane);



        /* Swing the edit buddy */
        editpane = gtk_vbox_new(FALSE, 0);

        
       	addbutton = gtk_button_new_with_label("Add");
       	rembutton = gtk_button_new_with_label("Remove");
       	tree = gtk_tree_new();
       	bbox = gtk_hbox_new(TRUE, 10);
       	tbox = gtk_scrolled_window_new(NULL, NULL);
       	/* Put the buttons in the box */
       	gtk_box_pack_start(GTK_BOX(bbox), addbutton, TRUE, TRUE, 10);
       	gtk_box_pack_start(GTK_BOX(bbox), rembutton, TRUE, TRUE, 10);

	gtk_tooltips_set_tip(tips, addbutton, "Add a new Buddy", "Penguin");
	gtk_tooltips_set_tip(tips, rembutton, "Remove selected Buddy", "Penguin");

       	/* And the boxes in the box */
       	gtk_box_pack_start(GTK_BOX(editpane), tbox, TRUE, TRUE, 5);
       	gtk_box_pack_start(GTK_BOX(editpane), bbox, FALSE, FALSE, 5);

	/* Handle closes right */

	

       	/* Finish up */
       	gtk_widget_show(addbutton);
       	gtk_widget_show(rembutton);
       	gtk_widget_show(tree);
       	gtk_widget_show(tbox);
       	gtk_widget_show(bbox);
	gtk_widget_show(editpane);


	/* Permit/Deny */

	permitpane = gtk_vbox_new(FALSE, 0);


        addpermbutton = gtk_button_new_with_label("Add");
        rempermbutton = gtk_button_new_with_label("Remove");
        
       	permtree = gtk_tree_new();
       	pbox = gtk_hbox_new(TRUE, 10);
       	xbox = gtk_scrolled_window_new(NULL, NULL);
       	/* Put the buttons in the box */
       	gtk_box_pack_start(GTK_BOX(pbox), addpermbutton, TRUE, TRUE, 10);
        gtk_box_pack_start(GTK_BOX(pbox), rempermbutton, TRUE, TRUE, 10);
        

	gtk_tooltips_set_tip(tips, addpermbutton, "Add person to permit/deny", "Penguin");
	gtk_tooltips_set_tip(tips, rempermbutton, "Remove person from permit/denycd ls", "Penguin");
       	/* And the boxes in the box */
       	gtk_box_pack_start(GTK_BOX(permitpane), xbox, TRUE, TRUE, 5);
       	gtk_box_pack_start(GTK_BOX(permitpane), pbox, FALSE, FALSE, 5);

	/* Handle closes right */

	

       	/* Finish up */
       	gtk_widget_show(addpermbutton);
        gtk_widget_show(rempermbutton);
       	gtk_widget_show(permtree);
       	gtk_widget_show(xbox);
       	gtk_widget_show(pbox);
	gtk_widget_show(permitpane);



        label = gtk_label_new("Online");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), buddypane, label);
        label = gtk_label_new("Edit Buddies");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), editpane, label);
	label = gtk_label_new("Permit");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), permitpane, label);

        gtk_widget_show_all(notebook);

	/* Pack things in the vbox */
        gtk_widget_show(vbox);


        gtk_widget_show(notebook);

        /* Enable buttons */
	
	gtk_signal_connect(GTK_OBJECT(imbutton), "clicked", GTK_SIGNAL_FUNC(show_im_dialog), buddies);
	gtk_signal_connect(GTK_OBJECT(infobutton), "clicked", GTK_SIGNAL_FUNC(info_callback), buddies);
	gtk_signal_connect(GTK_OBJECT(chatbutton), "clicked", GTK_SIGNAL_FUNC(chat_callback), buddies);
       	gtk_signal_connect(GTK_OBJECT(rembutton), "clicked", GTK_SIGNAL_FUNC(do_del_buddy), tree);
       	gtk_signal_connect(GTK_OBJECT(addbutton), "clicked", GTK_SIGNAL_FUNC(add_buddy_callback), NULL);
        gtk_signal_connect(GTK_OBJECT(addpermbutton), "clicked", GTK_SIGNAL_FUNC(add_perm_callback), NULL);
        gtk_signal_connect(GTK_OBJECT(rempermbutton), "clicked", GTK_SIGNAL_FUNC(do_del_perm), permtree);
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);

        gtk_container_add(GTK_CONTAINER(blist), vbox);

#ifndef USE_APPLET
        gtk_signal_connect(GTK_OBJECT(blist), "delete_event", GTK_SIGNAL_FUNC(do_quit), blist);
#else
	gtk_signal_connect(GTK_OBJECT(blist), "destroy", GTK_SIGNAL_FUNC(applet_destroy_buddy), NULL);
#endif


        
	parse_buddy_list(config, 1);
        gtk_window_set_title(GTK_WINDOW(blist), "Gaim - Buddy List");

        /* The edit tree */
        gtk_tree_clear_items(GTK_TREE(tree), 0, -1);
       	build_edit_tree();
#if (GTK_MINOR_VERSION > 1) || ((GTK_MINOR_VERSION > 0) && (GTK_MICRO_VERSION > 4))
       	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tbox), tree);
#else
       	gtk_container_add(GTK_CONTAINER(tbox), tree);
#endif
       	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tbox),
                                       GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);


        /* The permit tree */
        gtk_tree_clear_items(GTK_TREE(permtree), 0, -1);
       	build_permit_tree();
#if (GTK_MINOR_VERSION > 1) || ((GTK_MINOR_VERSION > 0) && (GTK_MICRO_VERSION > 4))
       	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(xbox), permtree);
#else
       	gtk_container_add(GTK_CONTAINER(xbox), permtree);
#endif
       	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(xbox),
                                       GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);

        update_button_pix();
        gtk_widget_show(blist);
}
