/*
 * gaim AOL Instant Messenger Client
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#ifdef USE_XMHTML
#include <gtk-xmhtml/gtk-xmhtml.h>
#endif
#include "aim.h"
#include "pixmaps/bold.xpm"
#include "pixmaps/italic.xpm"
#include "pixmaps/small.xpm"
#include "pixmaps/normal.xpm"
#include "pixmaps/big.xpm"
#include "pixmaps/speaker.xpm"
#include "pixmaps/aimicon2.xpm"
#include "pixmaps/wood.xpm"
#include "pixmaps/palette.xpm"

int state_lock=0;
int popup_windows;
int extrasounds;
GdkPixmap *dark_icon_pm = NULL;
GdkBitmap *dark_icon_bm = NULL;

struct away_message *awaymessage;
struct conversation *cnvroot;

void check_everything(GtkWidget *entry);


/*------------------------------------------------------------------------*/
/*  Helpers                                                               */
/*------------------------------------------------------------------------*/

void quiet_set(GtkWidget *tb, int state)
{
	state_lock=1;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tb), state);
	state_lock=0;
}


void set_state_lock(int i)
{
        state_lock = i;
}

static char *date()
{
	static char date[80];
	time_t tme;
	time(&tme);
	strftime(date, sizeof(date), "%H:%M:%S", localtime(&tme));
	return date;
}

struct conversation *new_conversation(char *name)
{
        struct conversation *c;

        c = find_conversation(name);
	
        if (c != NULL)
                return c;

        c = (struct conversation *)g_new(struct conversation, 1);
        g_snprintf(c->name, sizeof(c->name), "%s", name);
        show_conv(c);
        c->next = cnvroot;
        cnvroot = c;
        return c;
}


struct conversation *find_conversation(char *name)
{
        char *cuser = g_malloc(64);
        struct conversation *c, *t;
        c = t = cnvroot;
        strcpy(cuser, condense(name));

        while(c) {
                if(!strcasecmp(cuser, condense(c->name))) {
                        g_free(cuser);
                        return c;
                }
                t = c;
                c = c->next;
        }
        g_free(cuser);
        return NULL;
}

// ---------------------------------------------------
// Function to remove a log file entry
// ---------------------------------------------------

void rm_log(struct log_conversation *a)
{
        struct log_conversation *b = logroot, *c = logroot;
        struct conversation *cnv = find_conversation(a->name);
        char buf[128];
        
        while (b) {
                if (b == a) {
                        c->next = b->next;
                        if (b == logroot) {
                                logroot = b->next;
                        }

                        g_free(b);

                        save_prefs();

                        if (cnv) {
                                g_snprintf(buf, sizeof(buf), CONVERSATION_TITLE, cnv->name);
                                gtk_window_set_title(GTK_WINDOW(cnv->window), buf);
                        }
                                
                        return;
                }

                c = b;
                b = b->next;
        }
}

struct log_conversation *find_log_info(char *name)
{
	char *pname = g_malloc(64);
	struct log_conversation *l, *ll;
	l = ll = logroot;
	strcpy(pname, condense(name));

	while(l) {
		if (!strcasecmp(pname, condense(l->name))) {
			g_free(pname);
			return l;
		}
		ll = l;
		l = l->next;
	}
	g_free(pname);
	return NULL;
}

void delete_conversation(struct conversation *cnv)
{
        struct conversation *c, *t;
	if (!c)
		return;

        if (cnv == cnvroot) {
                cnvroot = cnvroot->next;
                return;
        }

        t = cnvroot;
        c = cnvroot->next;
        
        while(c) {
                if (c == cnv) {
                        t->next = c->next;
                        g_free(cnv);
                        return;
                }
                t = c;
                c = c->next;
        }
}

int is_logging(char *name)
{
        struct log_conversation *l;
	char *who = g_malloc(80);

	strcpy(who, condense(name));

        l = logroot;

	while (l) {
		if (!strcasecmp(who, condense(l->name))) {
			g_free(who);
			return 1;
		}
		l = l->next;
	}

	g_free(who);
	
	return 0;

}

/*------------------------------------------------------------------------*/
/*  Callbacks                                                             */
/*------------------------------------------------------------------------*/

void toggle_loggle(GtkWidget *w, struct conversation *p)
{
        if (state_lock)
                return;
        
        if (is_logging(p->name)) 
                rm_log(find_log_info(p->name));
        else 
		show_log_dialog(p->name);
}


static int close_callback(GtkWidget *widget, struct conversation *c)
{
        gtk_widget_destroy(c->window);
        delete_conversation(c);
        return TRUE;
}

static int delete_event(GtkWidget *w, void *dummy)
{
        struct conversation *c = (struct conversation *)gtk_object_get_user_data(GTK_OBJECT(w));
	gtk_widget_destroy(w);
	delete_conversation(c);
	return TRUE;
}
	

static void color_callback(GtkWidget *widget, struct conversation *c)
{
	// show_color_dialog(c); 
	gtk_widget_grab_focus(c->entry);
}

static void add_callback(GtkWidget *widget, struct conversation *c)
{
	GtkWidget *label;

	if (is_buddy(c->name)) {
#ifdef DEBUG
		printf("Removing '%s' from buddylist.\n", c->name);
#endif
		remove_person(find_category(c->name)->name, c->name);
		build_edit_tree();
		gtk_label_set_text(GTK_LABEL(GTK_BIN(c->add_button)->child), "Add");
	}
	else
	{
        	show_add_buddy(c->name, NULL);
	}

	gtk_widget_grab_focus(c->entry);
}


static void block_callback(GtkWidget *widget, struct conversation *c)
{
        show_add_perm(c->name);
	gtk_widget_grab_focus(c->entry);
}

static void warn_callback(GtkWidget *widget, struct conversation *c)
{
        show_warn_dialog(c->name);
	gtk_widget_grab_focus(c->entry);
}

static void info_callback(GtkWidget *widget, struct conversation *c)
{
	char *send = g_malloc(256);
	g_snprintf(send, 255, "toc_get_info %s", c->name);
	sflap_send(send, strlen(send), TYPE_DATA);
	g_free(send);
	gtk_widget_grab_focus(c->entry);
}

static void send_callback(GtkWidget *widget, struct conversation *c)
{
	char *buf = g_malloc(BUF_LONG);
	char *buf2 = g_malloc(BUF_LONG);

        strncpy(buf, gtk_entry_get_text(GTK_ENTRY(c->entry)), BUF_LONG / 2);
        if (!strlen(buf))
                return;
        

	write_to_conv(c->text, ourname, buf, WFLAG_SEND);

	gtk_entry_set_text(GTK_ENTRY(c->entry), "");

	escape_text(buf);
	g_snprintf(buf2, BUF_LONG / 2, "toc_send_im %s \"%s\"", condense(c->name), buf);
	sflap_send(buf2, -1, TYPE_DATA);
	quiet_set(c->bold, FALSE);
	quiet_set(c->italic, FALSE);
	quiet_set(c->palette, FALSE);
	if (c->makesound && extrasounds)
		play_sound(SEND);


	if (awaymessage != NULL) {
		do_im_back();
	}


	gtk_widget_grab_focus(c->entry);

        g_free(buf);
        g_free(buf2);

}

static int
entry_key_pressed(GtkWidget *w, GtkWidget *entry)
{
        check_everything(w);
        return TRUE;
}

/*------------------------------------------------------------------------*/
/*  HTML-type stuff                                                       */
/*------------------------------------------------------------------------*/

int count_tag(GtkWidget *entry, char *s1, char *s2)
{
	char *p1, *p2;
	int res=0;
	char *tmp, *tmpo, h;
	tmpo = gtk_entry_get_text(GTK_ENTRY(entry));
	h = tmpo[GTK_EDITABLE(entry)->current_pos];
	tmpo[GTK_EDITABLE(entry)->current_pos]='\0';
	tmp=tmpo;
	do {
		p1 = strstr(tmp, s1);
		p2 = strstr(tmp, s2);
		if (p1 && p2) {
			if (p1 < p2) {
				res=1;
				tmp = p1 +strlen(s1);
			} else if (p2 < p1) {
				res = 0;
				tmp = p2 + strlen(s2);
			}
		} else {
			if (p1) {
				res = 1;
				tmp = p1 + strlen(s1);
			} else if (p2) {
				res = 0;
				tmp = p2 + strlen(s2);
			}
		}
	} while (p1 || p2);
	tmpo[GTK_EDITABLE(entry)->current_pos]=h;
	return res;
}


int invert_tags(GtkWidget *entry, char *s1, char *s2, int really)
{
	int start = GTK_EDITABLE(entry)->selection_start_pos;
	int finish = GTK_EDITABLE(entry)->selection_end_pos;
	char *s;
	
	s = gtk_entry_get_text(GTK_ENTRY(entry));
	if (!strncasecmp(&s[start], s1, strlen(s1)) &&
	    !strncasecmp(&s[finish - strlen(s2)], s2, strlen(s2))) {
	   	if (really) {
	    		gtk_editable_delete_text(GTK_EDITABLE(entry), start, start + strlen(s1));
			gtk_editable_delete_text(GTK_EDITABLE(entry), finish - strlen(s2) - strlen(s1), finish - strlen(s1));
		}
		return 1;
	}
	return 0;
}


void remove_tags(GtkWidget *entry, char *tag)
{
	char *s;
	char *t;
	int start = GTK_EDITABLE(entry)->selection_start_pos;
	int finish = GTK_EDITABLE(entry)->selection_end_pos;
	s = gtk_entry_get_text(GTK_ENTRY(entry));
	t = s;
	while((t = strstr(t, tag))) {
		if (((t-s) < finish) && ((t-s) >= start)) 
			gtk_editable_delete_text(GTK_EDITABLE(entry), (t-s), (t-s) + strlen(tag));
		else t++;
	}
}

static void surround(GtkWidget *entry, char *pre, char *post)
{
	int pos = GTK_EDITABLE(entry)->current_pos;
	int dummy;
	int start, finish;
	if (GTK_EDITABLE(entry)->has_selection) {
		remove_tags(entry, pre);
		remove_tags(entry, post);
		start = GTK_EDITABLE(entry)->selection_start_pos;
		finish = GTK_EDITABLE(entry)->selection_end_pos;
		if (start > finish) {
			dummy = finish;
			finish = start;
			start = dummy;
		}
		dummy = start;
		gtk_editable_insert_text(GTK_EDITABLE(entry), pre, strlen(pre), &dummy);
		dummy = finish + strlen(pre);
		gtk_editable_insert_text(GTK_EDITABLE(entry), post, strlen(post), &dummy);
		gtk_editable_select_region(GTK_EDITABLE(entry), start, finish + strlen(pre) + strlen(post));
		
	} else {
		gtk_editable_insert_text(GTK_EDITABLE(entry), pre, strlen(pre), &pos);
		gtk_editable_insert_text(GTK_EDITABLE(entry), post, strlen(post), &dummy);
		gtk_entry_set_position(GTK_ENTRY(entry), pos);
	}
	gtk_widget_grab_focus(entry);
}

static void advance_past(GtkWidget *entry, char *pre, char *post)
{
	char *s, *s2;
	int pos;
	if (invert_tags(entry, pre, post, 1))
		return;
	s = gtk_entry_get_text(GTK_ENTRY(entry));
	pos = GTK_EDITABLE(entry)->current_pos;
#ifdef DEBUG
	printf("Currently at %d\n",pos);
#endif
	s2= strstr(&s[pos], post);
	if (s2)
		pos = s2 - s + strlen(post);
	else
		pos=-1;
#ifdef DEBUG
	printf("Setting position to %d\n",pos);
#endif
	gtk_entry_set_position(GTK_ENTRY(entry), pos);
	gtk_widget_grab_focus(entry);
}

static void toggle_color(GtkWidget *color, GtkWidget *entry)
{
        if (state_lock)
                return;
        if (GTK_TOGGLE_BUTTON(color)->active)
		show_color_dialog(entry, color);
        else
                advance_past(entry, "<FONT COLOR>", "</FONT>");
}

static void do_bold(GtkWidget *bold, GtkWidget *entry)
{
	if (state_lock)
		return;
	if (GTK_TOGGLE_BUTTON(bold)->active)
		surround(entry, "<B>","</B>");
	else
		advance_past(entry, "<B>", "</B>");
}

static void do_italic(GtkWidget *italic, GtkWidget *entry)
{
	if (state_lock)
		return;
	if (GTK_TOGGLE_BUTTON(italic)->active)
		surround(entry, "<I>","</I>");
	else
		advance_past(entry, "<I>", "</I>");
}

static void do_small(GtkWidget *small, GtkWidget *entry)
{
	if (state_lock)
		return;
	surround(entry, "<FONT SIZE=\"+1\">","</FONT>");
}

static void do_normal(GtkWidget *normal, GtkWidget *entry)
{
	surround(entry, "<FONT SIZE=\"+3\">","</FONT>");
}

static void do_big(GtkWidget *big, GtkWidget *entry)
{
	surround(entry, "<FONT SIZE=\"+5\">","</FONT>");
}

void check_everything(GtkWidget *entry)
{
	struct conversation *c;
	c = (struct conversation *)gtk_object_get_user_data(GTK_OBJECT(entry));
	if (!c) return;
	if (invert_tags(entry, "<B>", "</B>", 0))
		quiet_set(c->bold, TRUE);
	else if (count_tag(entry, "<B>", "</B>")) 
		quiet_set(c->bold, TRUE);
	else
		quiet_set(c->bold,FALSE);
	if (invert_tags(entry, "<I>", "</I>", 0))
		quiet_set(c->italic, TRUE);
	else if (count_tag(entry, "<I>", "</I>"))  
		quiet_set(c->italic, TRUE);
	else
		quiet_set(c->italic, FALSE);
     
	if (invert_tags(entry, "<FONT COLOR", "</FONT>", 0))
                quiet_set(c->palette, TRUE);
        else if (count_tag(entry, "<FONT COLOR", "</FONT>"))
                quiet_set(c->palette, TRUE);
        else
                quiet_set(c->palette, FALSE);

   
}


static void fix_text(GtkWidget *text)
{
	float f;
#if 0
	printf("Placing at (%f - %f) = %f\n",GTK_TEXT(text)->vadj->upper, (float)text->allocation.height, f);
#endif
	f = GTK_TEXT(text)->vadj->upper;
	f = f - text->allocation.height;
	if (f < 0)
		f=0;
	/* Special case */
	else if (text->allocation.height == 1) f = 0;
	while(gtk_events_pending())
		gtk_main_iteration();
	gtk_adjustment_set_value(GTK_TEXT(text)->vadj, f);
}


/*------------------------------------------------------------------------*/
/*  Takin care of the window..                                            */
/*------------------------------------------------------------------------*/


#ifdef USE_XMHTML
void write_to_conv(GtkWidget *html, char *who, char *what, int flags)
#else
void write_to_conv(GtkWidget *text, char *who, char *what, int flags)
#endif
{
	char *buf = g_malloc(BUF_LONG);
	char *buf2 = g_malloc(BUF_LONG);
        struct conversation *c = (struct conversation *)gtk_object_get_user_data(GTK_OBJECT(text));
	struct log_conversation *l;
	FILE *fd;
	
 

        
#ifndef USE_XMHTML
	
	GdkColor *colour;  
	static GdkColormap *map;
	
	map = gdk_window_get_colormap(c->window->window);
	colour = (GdkColor *)g_new(GdkColor, 1);
	if (flags & WFLAG_SEND) {
		colour->red = 0;
		colour->green = 0;
		colour->blue = 65535;
	} else {
		colour->red = 65535;
		colour->green = 0;
		colour->blue = 0;
	}
	gdk_color_alloc(map, colour);

	if (flags & WFLAG_AUTO)
		sprintf(buf2, "%s", AUTO_RESPONSE);
	else
		buf2[0]=0; /* sprintf(buf2, ""); */

	if (show_time)
		g_snprintf(buf, BUF_LONG, "%s %s: %s", date(), who, buf2);
	else
		g_snprintf(buf, BUF_LONG, "%s: %s", who, buf2);
	
	gtk_text_set_point(GTK_TEXT(text), gtk_text_get_length(GTK_TEXT(c->text)));
	gtk_text_freeze(GTK_TEXT(text));
	gtk_text_insert(GTK_TEXT(text), getfont(1,0,0,3), colour, NULL, buf, strlen(buf));
	html_print(text, what);
	gtk_text_thaw(GTK_TEXT(text));
	fix_text(text);
#else
	char colour[10];
	
	if (flags & WFLAG_RECV)
		colour = "#0000ff";
	else
		colour = "#ff0000";

	if (flags & WFLAG_AUTO)
		sprintf(buf2, "%s", AUTO_RESPONSE);
	else
		buf2[0]=0; /* sprintf(buf2, ""); */

	if (show_time) 
		g_snprintf(buf, sizeof(buf), "<FONT COLOR=\"%s\"><B>%s %s: %s</B></FONT> ", colour, date(), who, buf2);
	else
		g_snprintf(buf, sizeof(buf), "<FONT COLOR=\"%s\"><B>%s: %s</B></FONT> ", colour, who, buf2);
	html_write(c->text, buf);
	html_write(c->text, what);
	html_write(c->text,"<BR>");
	html_update(c->text);

#endif

	if (l = find_log_info(c->name)) {
		fd = fopen(l->filename, "a");
                if (show_time) fprintf(fd, "%s ", date());
                fprintf(fd, "%s: %s\n", who, what);
                fclose(fd);
	}
}
	



void conv_write(struct conversation *c, char *message, char *away)
{
	if (!strncasecmp(away, "T", 1))
		write_to_conv(c->text, c->name, message, WFLAG_RECV | WFLAG_AUTO);
	else
		write_to_conv(c->text, c->name, message, WFLAG_RECV);
	

	if (!gdk_window_is_viewable(c->window->window)) {

                if (dark_icon_pm == NULL)
                        dark_icon_pm = gdk_pixmap_create_from_xpm_d(c->window->window, &dark_icon_bm,
                                                                    NULL, (gchar **)aimicon2_xpm);
		gdk_window_set_icon(c->window->window, NULL, dark_icon_pm, dark_icon_bm);
	}


        if (popup_windows)
                gdk_window_raise(c->window->window);


	if (c->makesound && extrasounds)
		play_sound(RECEIVE);
}

void show_conv(struct conversation *c)
{
	GtkWidget *win;
        char buf[256];
        GtkWidget *text;
#ifndef USE_XMHTML
        GtkWidget *sw;
#endif
	GtkWidget *send;
	GtkWidget *info;
        GtkWidget *warn;
        GtkWidget *block;
	GtkWidget *color;
	GtkWidget *close;
	GtkWidget *entry;
	GtkWidget *toolbar;
	GtkWidget *tbox;
	GtkWidget *bbox;
        GtkWidget *vbox;
        GtkWidget *add;
        GdkPixmap *small_i, *normal_i, *big_i, *bold_i, *italic_i, *speaker_i, *wood_i, *palette_i;
        GtkWidget *small_p, *normal_p, *big_p, *bold_p, *italic_p, *speaker_p, *wood_p, *palette_p;
        GtkWidget *small, *normal, *big, *bold, *italic, *speaker, *wood, *palette;
	GdkBitmap *mask;
	
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_realize(win);
        aol_icon(win->window);

        c->window = win;
        
	send = gtk_button_new_with_label("Send");
	info = gtk_button_new_with_label("Info");
	warn = gtk_button_new_with_label("Warn");
	color = gtk_button_new_with_label("Color");
	close = gtk_button_new_with_label("Close");
	if (is_buddy(c->name)) {
       		add = gtk_button_new_with_label("Remove");
	}
	else {
		add = gtk_button_new_with_label("Add");
	}
        block = gtk_button_new_with_label("Block");
#ifdef USE_XMHTML
	text = gtk_xmhtml_new();
#else
	text = gtk_text_new(NULL, NULL);
	GTK_OBJECT(text)->flags &= ~GTK_CAN_FOCUS;
#endif
	bbox = gtk_hbox_new(TRUE, 0);
	tbox = gtk_hbox_new(FALSE, 0);
	vbox = gtk_vbox_new(FALSE, 0);
	entry = gtk_entry_new();

	gtk_widget_realize(win);
	/* Toolbar */
	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	palette_i = gdk_pixmap_create_from_xpm_d (win->window, &mask,
             &win->style->white, palette_xpm );
	palette_p = gtk_pixmap_new(palette_i, mask);
	gtk_widget_show(palette_p);

	wood_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask, 
	     &win->style->white, wood_xpm );
	wood_p = gtk_pixmap_new(wood_i, mask);
	gtk_widget_show(wood_p);
	speaker_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, speaker_xpm );
	speaker_p = gtk_pixmap_new(speaker_i, mask);
	gtk_widget_show(speaker_p);
	c->makesound=1;
	bold_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, bold_xpm );
	bold_p = gtk_pixmap_new(bold_i, mask);
	gtk_widget_show(bold_p);
	italic_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, italic_xpm );
	italic_p = gtk_pixmap_new(italic_i, mask);
	gtk_widget_show(italic_p);
	small_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, small_xpm );
	small_p = gtk_pixmap_new(small_i, mask);
	gtk_widget_show(small_p);
	normal_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, normal_xpm );
	normal_p = gtk_pixmap_new(normal_i, mask);
	gtk_widget_show(normal_p);
	big_i = gdk_pixmap_create_from_xpm_d ( win->window, &mask,
             &win->style->white, big_xpm );
	big_p = gtk_pixmap_new(big_i, mask);
	gtk_widget_show(big_p);


	bold = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
	                                  GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL,
					  "Bold", "Bold Text", "Bold", bold_p,
					  GTK_SIGNAL_FUNC(do_bold), entry);
	italic = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		                            GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
					    NULL, "Italics", "Italics Text",
					    "Italics", italic_p, GTK_SIGNAL_FUNC(do_italic), entry);
	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
	small = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "Small", "Decrease font size", "Small", small_p, GTK_SIGNAL_FUNC(do_small), entry);
	normal = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "Normal", "Normal font size", "Normal", normal_p, GTK_SIGNAL_FUNC(do_normal), entry);
	big = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "Big", "Increase font size", "Big", big_p, GTK_SIGNAL_FUNC(do_big), entry);
	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
	palette = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
					    GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
					    NULL, "Color", "Text Color",
				 	    "Color", palette_p, GTK_SIGNAL_FUNC(toggle_color), entry); 
	wood = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
					    GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
					    NULL, "Logging", "Enable logging",
                                          "Logging", wood_p, GTK_SIGNAL_FUNC(toggle_loggle), c);
        speaker = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		                            GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
					    NULL, "Sound", "Enable sounds",
					    "Sound", speaker_p, GTK_SIGNAL_FUNC(set_option), &c->makesound);
	c->makesound=0;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(speaker), TRUE);

        state_lock = 1;
	if (is_logging(c->name))
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(wood), TRUE);
	else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(wood), FALSE);
        state_lock = 0;
        
	gtk_widget_show(toolbar);
	
#ifdef USE_XMHTML
	c->text = html;
#else
	c->text = text;
#endif
	c->entry = entry;
	c->bold = bold;
        c->italic = italic;
        c->log_button = wood;
	c->palette = palette;
        c->add_button = add;
        
	gtk_object_set_user_data(GTK_OBJECT(entry), c);

	

	gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(send_callback),c);
        /* Text box */








#ifdef USE_XMHTML
//FIXME       gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), html);
//        gtk_box_pack_start(GTK_BOX(tbox), sw, FALSE, FALSE, 0);
//        gtk_widget_set_usize(html, 320, 150);
 
#else
        sw = gtk_scrolled_window_new (NULL, NULL);
        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                        GTK_POLICY_NEVER,
                                        GTK_POLICY_ALWAYS);
        gtk_widget_show (sw);

        gtk_container_add (GTK_CONTAINER (sw), text);
        gtk_widget_show (text);
        gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
        gtk_widget_set_usize(sw, 320, 150);
#endif



        

	/* Ready and pack buttons */
	gtk_object_set_user_data(GTK_OBJECT(win), c);
	gtk_object_set_user_data(GTK_OBJECT(close), c);
	gtk_signal_connect(GTK_OBJECT(close), "clicked", GTK_SIGNAL_FUNC(close_callback), c);
	gtk_signal_connect(GTK_OBJECT(send), "clicked", GTK_SIGNAL_FUNC(send_callback), c);
	gtk_signal_connect(GTK_OBJECT(add), "clicked", GTK_SIGNAL_FUNC(add_callback), c);
        gtk_signal_connect(GTK_OBJECT(info), "clicked", GTK_SIGNAL_FUNC(info_callback), c);
        gtk_signal_connect(GTK_OBJECT(warn), "clicked", GTK_SIGNAL_FUNC(warn_callback), c);
        gtk_signal_connect(GTK_OBJECT(block), "clicked", GTK_SIGNAL_FUNC(block_callback), c);
	gtk_signal_connect(GTK_OBJECT(color), "clicked", GTK_SIGNAL_FUNC(color_callback), c);
       
	gtk_box_pack_start(GTK_BOX(bbox), send, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(bbox), info, TRUE, TRUE, 5);
        gtk_box_pack_start(GTK_BOX(bbox), warn, TRUE, TRUE, 5);
        gtk_box_pack_start(GTK_BOX(bbox), block, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(bbox), color, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(bbox), add, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(bbox), close, TRUE, TRUE, 5);
	
	/* pack and fill the rest */

	
	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);

	


	gtk_widget_show(send);
	gtk_widget_show(info);
	gtk_widget_show(warn);
	// gtk_widget_show(color);
	gtk_widget_show(close);	
        gtk_widget_show(add);
        gtk_widget_show(block);
	gtk_widget_show(bbox);
	gtk_widget_show(vbox);
	gtk_widget_show(tbox);
	gtk_widget_show(entry);
	gtk_object_set_user_data(GTK_OBJECT(text), c);
	gtk_widget_show(text);
	
	gtk_container_add(GTK_CONTAINER(win),vbox);
        gtk_container_border_width(GTK_CONTAINER(win), 10);

	if (is_logging(c->name))
		g_snprintf(buf, sizeof(buf), LOG_CONVERSATION_TITLE, c->name);
	else
		g_snprintf(buf, sizeof(buf), CONVERSATION_TITLE, c->name);
	gtk_window_set_title(GTK_WINDOW(win), buf);
	gtk_window_set_focus(GTK_WINDOW(win),entry);
	c->text = text;

	gtk_signal_connect(GTK_OBJECT(win), "delete_event", GTK_SIGNAL_FUNC(delete_event), c);
	gtk_signal_connect(GTK_OBJECT(entry), "key_press_event", GTK_SIGNAL_FUNC(entry_key_pressed), NULL);
	
	gtk_widget_show(win);
	
}


