/*                                  l
 * gaim AOL Instant Messenger Client
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include "aim.h"

struct conversation *cnvroot;
struct category *croot;
GList *permit, *deny;
struct buddy_pounce *bproot;
struct log_conversation *logroot;

static GtkWidget *imdialog = NULL; //I only want ONE of these :) */
static GList *dialogwindows = NULL;
static GtkWidget *colordialog, *exportdialog, *importdialog, *logdialog;

char *test;
char *test2;
GtkWidget *entry;

struct warning {
        GtkWidget *window;
        GtkWidget *anon;
        char *who;
};

struct addbuddy {
        GtkWidget *window;
        GtkWidget *combo;
        GtkWidget *entry;
};

struct addperm {
        GtkWidget *window;
        GSList *buttons;
        GtkWidget *entry;
};

struct addbp {
        GtkWidget *window;
        GtkWidget *nameentry;
        GtkWidget *messentry;
};



/*------------------------------------------------------------------------*/
/*  Destroys                                                              */
/*------------------------------------------------------------------------*/


static void destroy_dialog(GtkWidget *w, GtkWidget *w2)
{
        GtkWidget *dest;
        
        if (!GTK_IS_WIDGET(w2))
                dest = w;
        else
                dest = w2;

        if (dest == imdialog)
		imdialog = NULL;

	if (dest == exportdialog)
		exportdialog = NULL;

	if (dest == importdialog)
		importdialog = NULL;

	if (dest == logdialog)
		logdialog = NULL;

	if (dest == colordialog)
		colordialog = NULL;

        dialogwindows = g_list_remove(dialogwindows, dest);
        gtk_widget_destroy(dest);

}


void destroy_all_dialogs()
{
        GList *d = dialogwindows;
        while(d) {
                destroy_dialog(NULL, d->data);
                d = d->next;
                printf("Dest!\n");
        }
        
        g_list_free(dialogwindows);
        dialogwindows = NULL;

        if (imdialog) {
                destroy_dialog(NULL, imdialog);
                imdialog = NULL;
        }
        
	if (colordialog) {
		destroy_dialog(NULL, colordialog);
		colordialog = NULL;
	}

        if (exportdialog) {
                destroy_dialog(NULL, exportdialog);
                exportdialog = NULL;
        }

        if (importdialog) {
                destroy_dialog(NULL, exportdialog);
                importdialog = NULL;
        }
	
	if (logdialog) {
		destroy_dialog(NULL, logdialog);
		logdialog = NULL;
	}

}

static void surround(GtkWidget *entry, char *pre, char *post)
{
        int pos = GTK_EDITABLE(entry)->current_pos;
        int dummy;
        int start, finish;
        if (GTK_EDITABLE(entry)->has_selection) {
                remove_tags(entry, pre);
                remove_tags(entry, post);
                start = GTK_EDITABLE(entry)->selection_start_pos;
                finish = GTK_EDITABLE(entry)->selection_end_pos;
                if (start > finish) {
                        dummy = finish;
                        finish = start;
                        start = dummy;
                }
                dummy = start;
                gtk_editable_insert_text(GTK_EDITABLE(entry), pre, strlen(pre),
&dummy);
                dummy = finish + strlen(pre);
                gtk_editable_insert_text(GTK_EDITABLE(entry), post, strlen(post), &dummy);
                gtk_editable_select_region(GTK_EDITABLE(entry), start, finish +
strlen(pre) + strlen(post));



        } else {
                gtk_editable_insert_text(GTK_EDITABLE(entry), pre, strlen(pre),
&pos);
                gtk_editable_insert_text(GTK_EDITABLE(entry), post, strlen(post), &dummy);
                gtk_entry_set_position(GTK_ENTRY(entry), pos);
        }
        gtk_widget_grab_focus(entry);
}


static void do_warn(GtkWidget *widget, struct warning *w)
{
        char *send = g_malloc(256);
        g_snprintf(send, 255, "toc_evil %s %s", w->who,
                   (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w->anon))) ?
                   "anon" : "norm");
        printf("%s\n", send);
        sflap_send(send, -1, TYPE_DATA);
        g_free(send);
        
        destroy_dialog(NULL, w->window);
}


void show_warn_dialog(char *who)
{
	GtkWidget *cancel;
	GtkWidget *warn;
	GtkWidget *label;
	GtkWidget *vbox;
        GtkWidget *bbox;
        struct warning *w = g_new(struct warning, 1);
        
        char *buf = g_malloc(128);
        w->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_widget_realize(w->window);
        aol_icon(w->window->window);
        dialogwindows = g_list_prepend(dialogwindows, w->window);
        cancel = gtk_button_new_with_label("Cancel");
        warn = gtk_button_new_with_label("Warn");
        bbox = gtk_hbox_new(TRUE, 10);
        vbox = gtk_vbox_new(FALSE, 5);

        /* Put the buttons in the box */
        gtk_box_pack_start(GTK_BOX(bbox), warn, TRUE, TRUE, 10);
        gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);

        g_snprintf(buf, 127, "Do you really want to warn %s?", who);
        label = gtk_label_new(buf);
        gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);
        w->anon = gtk_check_button_new_with_label("Warn anonymously?");
        gtk_box_pack_start(GTK_BOX(vbox), w->anon, TRUE, TRUE, 0);

        label = gtk_label_new("Anonymous warnings are less harsh.");
        gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        w->who = who;
		
        gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);

        /* Handle closes right */
        gtk_signal_connect(GTK_OBJECT(w->window), "delete_event",
                           GTK_SIGNAL_FUNC(destroy_dialog), w->window);
        gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
                           GTK_SIGNAL_FUNC(destroy_dialog), w->window);
        gtk_signal_connect(GTK_OBJECT(warn), "clicked",
                           GTK_SIGNAL_FUNC(do_warn), w);
        /* Finish up */
        gtk_widget_show(warn);
        gtk_widget_show(cancel);
        gtk_widget_show(w->anon);
        gtk_widget_show(bbox);
        gtk_widget_show(vbox);
        gtk_window_set_title(GTK_WINDOW(w->window), "Gaim - Warn user?");
        gtk_container_add(GTK_CONTAINER(w->window), vbox);
        gtk_widget_show(w->window);
}


/*------------------------------------------------------------------------*/
/*  The dialog for getting an error                                       */
/*------------------------------------------------------------------------*/

void show_error_dialog(char *c)
{

	int no = atoi(c);
	char *w = strtok(NULL, ":");
	char buf[256];
	char buf2[32];
	GtkWidget *d;
	GtkWidget *label;
	GtkWidget *close;
	
	
        switch(no) {
        case 69:
                g_snprintf(buf, sizeof(buf), "Unable to write file %s.", w);
                break;
        case 169:
                g_snprintf(buf, sizeof(buf), "Unable to read file %s.", w);
                break;
        case 901:
                g_snprintf(buf, sizeof(buf), "%s not currently logged in.", w);
                break;
        case 902:
                g_snprintf(buf, sizeof(buf), "Warning of %s not allowed.", w);
                break;
        case 903:
                g_snprintf(buf, sizeof(buf), "A message has been dropped, you are exceeding the server speed limit.");
                break;
        case 950:
                g_snprintf(buf, sizeof(buf), "Chat in %s is not available.", w);
                break;
        case 960:
                g_snprintf(buf, sizeof(buf), "You are sending messages too fast to %s.", w);
                break;
        case 961:
                g_snprintf(buf, sizeof(buf), "You missed an IM from %s because it was too big.", w);
                break;
        case 962:
                g_snprintf(buf, sizeof(buf), "You missed an IM from %s because it was sent too fast.", w);
                break;
        case 970:
                g_snprintf(buf, sizeof(buf), "Failure.");
                break;
        case 971:
                g_snprintf(buf, sizeof(buf), "Too many matches.");
                break;
        case 972:
                g_snprintf(buf, sizeof(buf), "Need more qualifiers.");
                break;
        case 973:
                g_snprintf(buf, sizeof(buf), "Dir service temporarily unavailable.");
                break;
        case 974:
                g_snprintf(buf, sizeof(buf), "Email lookup restricted.");
                break;
        case 975:
                g_snprintf(buf, sizeof(buf), "Keyword ignored.");
                break;
        case 976:
                g_snprintf(buf, sizeof(buf), "No keywords.");
                break;
        case 977:
                g_snprintf(buf, sizeof(buf), "Language not supported.");
                break;
        case 978:
                g_snprintf(buf, sizeof(buf), "Country not supported.");
                break;
        case 979:
                g_snprintf(buf, sizeof(buf), "Failure unknown: %s.", w);
                break;
        case 980:
                g_snprintf(buf, sizeof(buf), "Incorrect nickname or password.");
                break;
        case 981:
                g_snprintf(buf, sizeof(buf), "The service is temporarily unavailable.");
                break;
        case 982:
                g_snprintf(buf, sizeof(buf), "Your warning level is currently too high to log in.");
                break;
        case 983:
                g_snprintf(buf, sizeof(buf), "You have been connecting and disconnecting too frequently.  Wait ten minutes and try again.  If you continue to try, you will need to wait even longer.");
                break;
        case 989:
                g_snprintf(buf, sizeof(buf), "An unknown signon error has occurred: %s.", w);
                break;
        default:
                g_snprintf(buf, sizeof(buf), "An unknown error, %d, has occured.  Info: %s", no, w);
	}
	
	d = gtk_dialog_new();

	label = gtk_label_new(buf);
	gtk_widget_show(label);
	close = gtk_button_new_with_label("Close");
	gtk_widget_show(close);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),
		label, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->action_area), 
		close, FALSE, FALSE, 5);
		
//	gtk_widget_set_usize(d, 200, 110);
/*	gtk_container_set_border_width(GTK_CONTAINER(d), 0);
*/
	g_snprintf(buf2, sizeof(buf2), "Gaim - Error %d", no);
	gtk_window_set_title(GTK_WINDOW(d), buf2);
	gtk_signal_connect(GTK_OBJECT(close), "clicked", GTK_SIGNAL_FUNC(destroy_dialog), d);
//	aol_icon(d->window);
	gtk_widget_show(d);
}



static void do_im(GtkWidget *widget, GtkWidget *imentry)
{
        char *who;
        struct conversation *c;

        who = condense(gtk_entry_get_text(GTK_ENTRY(imentry)));
        
	destroy_dialog(NULL, imdialog);
        imdialog = NULL;
        
        if (!strcasecmp(who, ""))
                return;


        c = find_conversation(who);

        if (c == NULL) {
                c = new_conversation(who);
        } else {
                gdk_window_raise(c->window->window);
        }
}

void show_im_dialog(GtkWidget *w, GtkWidget *w2)
{
        GtkWidget *cancel;
	GtkWidget *ok;
	GtkWidget *imentry;
        GtkWidget *vbox;
        GtkWidget *ebox;
        GtkWidget *bbox;
        GtkWidget *label;

        if (!imdialog) {

                imdialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
                gtk_widget_realize(imdialog);
                aol_icon(imdialog->window);
                cancel = gtk_button_new_with_label("Cancel");
                ok = gtk_button_new_with_label("OK");
                bbox = gtk_hbox_new(TRUE, 10);
                vbox = gtk_vbox_new(FALSE, 5);
                ebox = gtk_hbox_new(FALSE, 2);

                /* Put the buttons in the box */
                gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 10);
                gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);

                label = gtk_label_new("IM who: ");
                gtk_box_pack_start(GTK_BOX(ebox), label, TRUE, TRUE, 10);
                gtk_widget_show(label);

                imentry = gtk_entry_new();
                gtk_box_pack_start(GTK_BOX(ebox), imentry, TRUE, TRUE, 10);

                gtk_box_pack_start(GTK_BOX(vbox), ebox, FALSE, FALSE, 5);
                gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);

                /* Handle closes right */
		gtk_signal_connect(GTK_OBJECT(imentry), "activate",
				   GTK_SIGNAL_FUNC(do_im), imentry);
                gtk_signal_connect(GTK_OBJECT(imdialog), "destroy",
                                   GTK_SIGNAL_FUNC(destroy_dialog), imdialog);
                gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
                                   GTK_SIGNAL_FUNC(destroy_dialog), imdialog);
                        gtk_signal_connect(GTK_OBJECT(ok), "clicked",
                                           GTK_SIGNAL_FUNC(do_im), imentry);
                /* Finish up */
                gtk_widget_show(ok);
                gtk_widget_show(cancel);
                gtk_widget_show(ebox);
                gtk_widget_show(imentry);
                gtk_widget_show(bbox);
                gtk_widget_show(vbox);
                gtk_window_set_title(GTK_WINDOW(imdialog), "Gaim - IM user");
                gtk_container_add(GTK_CONTAINER(imdialog), vbox);
		gtk_widget_grab_focus(imentry);
        }
        gtk_widget_show(imdialog);
}


/*------------------------------------------------------------------------*/
/*  The dialog for adding buddies                                         */
/*------------------------------------------------------------------------*/

void do_add_buddy(GtkWidget *w, struct addbuddy *a)
{
	char *cat, *who;
	char buf[BUF_LONG];
	struct conversation *c;
        
	who = gtk_entry_get_text(GTK_ENTRY(a->entry));
	cat = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(a->combo)->entry));
        c = find_conversation(who);
        
        if (!add_person(cat,who)) {
                add_category(cat);
                add_person(cat,who);
        }

        if (c != NULL)
		gtk_label_set_text(GTK_LABEL(GTK_BIN(c->add_button)->child), "Remove");

        
        build_edit_tree();

        build_config(buf, sizeof(buf)/2);
        sflap_send(buf, -1, TYPE_DATA);

        g_snprintf(buf, sizeof(buf)/2, "toc_add_buddy %s", condense(who));
        sflap_send(buf, -1, TYPE_DATA);

        update_num_categories();

        destroy_dialog(NULL, a->window);
}


static GList *categories()
{
	GList *tmp=NULL;
	GtkWidget *li;
	struct category *c;
	c=croot;
	if (!c) {
		li=gtk_list_item_new_with_label("Buddies");
		gtk_widget_show(li);
		tmp = g_list_prepend(tmp, li);
	}
	while(c) {
		li=gtk_list_item_new_with_label(c->name);
		gtk_widget_show(li);
		tmp=g_list_prepend(tmp, li);
		c=c->next;
	}
	return tmp;
}


void show_add_buddy(char *buddy, char *group)
{
	GtkWidget *cancel;
	GtkWidget *add;
	GtkWidget *label;
	GtkWidget *bbox;
	GtkWidget *vbox;
        GtkWidget *topbox;
        struct addbuddy *a = g_new(struct addbuddy, 1);
        
        a->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_widget_realize(a->window);
        aol_icon(a->window->window);
        dialogwindows = g_list_prepend(dialogwindows, a->window);
        cancel = gtk_button_new_with_label("Cancel");
        add = gtk_button_new_with_label("Add");
        bbox = gtk_hbox_new(TRUE, 10);
        topbox = gtk_hbox_new(FALSE, 5);
        vbox = gtk_vbox_new(FALSE, 5);
        a->entry = gtk_entry_new();
        a->combo = gtk_combo_new();
        /* Fix the combo box */
        gtk_list_append_items(GTK_LIST(GTK_COMBO(a->combo)->list), categories());
        /* Put the buttons in the box */
        gtk_box_pack_start(GTK_BOX(bbox), add, TRUE, TRUE, 10);
        gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);

        label = gtk_label_new("Add");
        gtk_widget_show(label);
        gtk_box_pack_start(GTK_BOX(topbox), label, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), a->entry, FALSE, FALSE, 5);
        if (buddy != NULL)
                gtk_entry_set_text(GTK_ENTRY(a->entry), buddy);

        label = gtk_label_new("to group");
        gtk_widget_show(label);
        gtk_box_pack_start(GTK_BOX(topbox), label, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), a->combo, FALSE, FALSE, 5);

        if (group != NULL)
                gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(a->combo)->entry), group);

        /* And the boxes in the box */
        gtk_box_pack_start(GTK_BOX(vbox), topbox, TRUE, TRUE, 5);
        gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);

        /* Handle closes right */
        gtk_signal_connect(GTK_OBJECT(a->window), "destroy",
                           GTK_SIGNAL_FUNC(destroy_dialog), a->window);
        gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
                           GTK_SIGNAL_FUNC(destroy_dialog), a->window);
        gtk_signal_connect(GTK_OBJECT(add), "clicked",
                           GTK_SIGNAL_FUNC(do_add_buddy), a);
        gtk_signal_connect(GTK_OBJECT(a->entry), "activate",
                           GTK_SIGNAL_FUNC(do_add_buddy), a);
        /* Finish up */
        gtk_widget_show(add);
        gtk_widget_show(cancel);
        gtk_widget_show(a->combo);
        gtk_widget_show(a->entry);
        gtk_widget_show(topbox);
        gtk_widget_show(bbox);
        gtk_widget_show(vbox);
        gtk_window_set_title(GTK_WINDOW(a->window), "Gaim - Add Buddy");
        gtk_window_set_focus(GTK_WINDOW(a->window), a->entry);
        gtk_container_add(GTK_CONTAINER(a->window), vbox);
	gtk_widget_show(a->window);
}


/*------------------------------------------------------------------------*/
/*  The dialog for new buddy pounces                                      */
/*------------------------------------------------------------------------*/


void do_new_bp(GtkWidget *w, struct addbp *b)
{
        struct buddy_pounce *bp = g_new(struct buddy_pounce, 1);

        strcpy(bp->name, gtk_entry_get_text(GTK_ENTRY(b->nameentry)));
        strcpy(bp->message, gtk_entry_get_text(GTK_ENTRY(b->messentry)));
        
        bp->next = bproot;
        bproot = bp;

        do_bp_menu();
        
        
        destroy_dialog(NULL, b->window);
        g_free(b);
}


void show_new_bp()
{
	GtkWidget *cancel;
	GtkWidget *ok;
	GtkWidget *label;
	GtkWidget *bbox;
	
	GtkWidget *vbox;
        GtkWidget *topbox;

        struct addbp *b = g_new(struct addbp, 1);
        
        b->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_widget_realize(b->window);
        aol_icon(b->window->window);
        dialogwindows = g_list_prepend(dialogwindows, b->window);

        cancel = gtk_button_new_with_label("Cancel");
        ok = gtk_button_new_with_label("OK");
        bbox = gtk_hbox_new(TRUE, 10);
        topbox = gtk_hbox_new(FALSE, 5);
        vbox = gtk_vbox_new(FALSE, 5);
        b->nameentry = gtk_entry_new();
        b->messentry = gtk_entry_new();

        /* Put the buttons in the box */
        gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 10);
        gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);

        label = gtk_label_new("Buddy");
        gtk_widget_show(label);
        gtk_box_pack_start(GTK_BOX(topbox), label, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), b->nameentry, FALSE, FALSE, 5);
        label = gtk_label_new("Message");
        gtk_widget_show(label);
        gtk_box_pack_start(GTK_BOX(topbox), label, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), b->messentry, FALSE, FALSE, 5);


        /* And the boxes in the box */
        gtk_box_pack_start(GTK_BOX(vbox), topbox, TRUE, TRUE, 5);
        gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);

        /* Handle closes right */
        gtk_signal_connect(GTK_OBJECT(b->window), "destroy",
                           GTK_SIGNAL_FUNC(destroy_dialog), b->window);
        gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
                           GTK_SIGNAL_FUNC(destroy_dialog), b->window);
        gtk_signal_connect(GTK_OBJECT(ok), "clicked",
                           GTK_SIGNAL_FUNC(do_new_bp), b);
        gtk_signal_connect(GTK_OBJECT(b->messentry), "activate",
                           GTK_SIGNAL_FUNC(do_new_bp), b);
        /* Finish up */
        gtk_widget_show(ok);
        gtk_widget_show(cancel);
        gtk_widget_show(b->nameentry);
        gtk_widget_show(b->messentry);
        gtk_widget_show(topbox);
        gtk_widget_show(bbox);
        gtk_widget_show(vbox);
        gtk_window_set_title(GTK_WINDOW(b->window), "Gaim - New Buddy Pounce");
        gtk_window_set_focus(GTK_WINDOW(b->window), b->nameentry);
        gtk_container_add(GTK_CONTAINER(b->window), vbox);
	gtk_widget_show(b->window);
}


/*------------------------------------------------------------------------*/
/*  The dialog for adding to permit/deny                                  */
/*------------------------------------------------------------------------*/


static void do_add_perm(GtkWidget *w, struct addperm *p)
{

        char *who;
        char *name;
        int d = 0;
        char buf[BUF_LONG];
        GSList *buttons = p->buttons;


        who = gtk_entry_get_text(GTK_ENTRY(p->entry));

        name = g_malloc(strlen(who) + 2);
        g_snprintf(name, strlen(who) + 2, "%s", who);
        
        while(buttons) {
                if((int)gtk_object_get_user_data(GTK_OBJECT(buttons->data)) == 1) {
                        if (GTK_TOGGLE_BUTTON(buttons->data)->active)
                                d = 1;
                }
                buttons = buttons->next;
        }

        if (d) {
                deny = g_list_append(deny, name);
                g_snprintf(buf, sizeof(buf)/2, "toc_add_deny %s", condense(name));
                sflap_send(buf, -1, TYPE_DATA);
        } else {
                permit = g_list_append(permit, name);
                g_snprintf(buf, sizeof(buf)/2, "toc_add_permit %s", condense(name));
                sflap_send(buf, -1, TYPE_DATA);
        }



        build_permit_tree();



        build_config(buf, sizeof(buf)/2);

        sflap_send(buf, -1, TYPE_DATA);

        destroy_dialog(NULL, p->window);
}



void show_add_perm(char *who)
{
	GtkWidget *cancel;
	GtkWidget *add;
	GtkWidget *label;
	GtkWidget *bbox;
        GtkWidget *vbox;
        GtkWidget *rbox;
        GtkWidget *topbox;
        GtkWidget *which;
        struct addperm *p = g_new(struct addperm, 1);

        p->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_widget_realize(p->window);
        aol_icon(p->window->window);
        dialogwindows = g_list_prepend(dialogwindows, p->window);
        cancel = gtk_button_new_with_label("Cancel");
        add = gtk_button_new_with_label("Add");
        bbox = gtk_hbox_new(TRUE, 10);
        topbox = gtk_hbox_new(FALSE, 5);
        vbox = gtk_vbox_new(FALSE, 5);
        rbox = gtk_vbox_new(FALSE, 5);
        p->entry = gtk_entry_new();

        if (who != NULL)
                gtk_entry_set_text(GTK_ENTRY(p->entry), who);

        which = gtk_radio_button_new_with_label(NULL, "Deny");
        gtk_box_pack_start(GTK_BOX(rbox), which, FALSE, FALSE, 0);
        gtk_object_set_user_data(GTK_OBJECT(which), (int *)1);
        gtk_widget_show(which);

        which = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(which)), "Permit");
        gtk_box_pack_start(GTK_BOX(rbox), which, FALSE, FALSE, 0);
        gtk_object_set_user_data(GTK_OBJECT(which), (int *)2);
        gtk_widget_show(which);
                
        /* Put the buttons in the box */
        gtk_box_pack_start(GTK_BOX(bbox), add, TRUE, TRUE, 10);
        gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);
		
        label = gtk_label_new("Add");
        gtk_widget_show(label);
        gtk_box_pack_start(GTK_BOX(topbox), label, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), p->entry, FALSE, FALSE, 5);
        gtk_box_pack_start(GTK_BOX(topbox), rbox, FALSE, FALSE, 5);
        /* And the boxes in the box */
        gtk_box_pack_start(GTK_BOX(vbox), topbox, TRUE, TRUE, 5);
        gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 5);


        p->buttons = gtk_radio_button_group(GTK_RADIO_BUTTON(which));
        /* Handle closes right */
        gtk_signal_connect(GTK_OBJECT(p->window), "destroy",
                           GTK_SIGNAL_FUNC(destroy_dialog), p->window);
        gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
                           GTK_SIGNAL_FUNC(destroy_dialog), p->window);
        gtk_signal_connect(GTK_OBJECT(add), "clicked",
                           GTK_SIGNAL_FUNC(do_add_perm), p);
        gtk_signal_connect(GTK_OBJECT(p->entry), "activate",
                           GTK_SIGNAL_FUNC(do_add_perm), p);

        /* Finish up */
        gtk_widget_show(add);
        gtk_widget_show(cancel);
        gtk_widget_show(p->entry);
        gtk_widget_show(topbox);
        gtk_widget_show(bbox);
        gtk_widget_show(vbox);
        gtk_widget_show(rbox);
        gtk_window_set_title(GTK_WINDOW(p->window), "Gaim - Add Permit/Deny");
        gtk_window_set_focus(GTK_WINDOW(p->window), p->entry);
        gtk_container_add(GTK_CONTAINER(p->window), vbox);
        gtk_widget_show(p->window);
}


/*------------------------------------------------------------------------*/
/*  Function Called To Add A Log                                          */
/*------------------------------------------------------------------------*/

void do_log(GtkWidget *w, struct conversation *c)
{
	struct log_conversation *l;
        char buf[128];
	
	if (!find_log_info(c->name)) {
		l = (struct log_conversation *)g_new(struct log_conversation, 1);
		strcpy(l->name, c->name);
		strcpy(l->filename, gtk_file_selection_get_filename(GTK_FILE_SELECTION(logdialog)));
		l->next = logroot;
		logroot = l;
		g_snprintf(buf, sizeof(buf), LOG_CONVERSATION_TITLE, c->name);
		gtk_window_set_title(GTK_WINDOW(c->window), buf);
	}

	save_prefs();

	

	destroy_dialog(NULL, logdialog);
	logdialog = NULL;
}

static void cancel_log(GtkWidget *w, char *name)
{
	struct conversation *c = gtk_object_get_user_data(GTK_OBJECT(logdialog));
        set_state_lock(1);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(c->log_button), FALSE);
        set_state_lock(0);
	destroy_dialog(NULL, logdialog);
}

void show_log_dialog(char *bname)
{
	char *buf = g_malloc(BUF_LEN);
        struct conversation *c = find_conversation(bname);

	
	if (!logdialog) {
		logdialog = gtk_file_selection_new("Gaim - Log Conversation");
		
		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(logdialog));	

                gtk_object_set_user_data(GTK_OBJECT(logdialog), c);
		
		g_snprintf(buf, BUF_LEN - 1, "%s/%s.log", getenv("HOME"), bname);
		
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(logdialog), buf);
		gtk_signal_connect(GTK_OBJECT(logdialog), "delete_event", GTK_SIGNAL_FUNC(cancel_log), c);
		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(logdialog)->ok_button), "clicked", GTK_SIGNAL_FUNC(do_log), c);
		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(logdialog)->cancel_button), "clicked", GTK_SIGNAL_FUNC(cancel_log), c);
	}

	gtk_widget_show(logdialog);
	gdk_window_raise(logdialog->window);
}

/*------------------------------------------------------*/
/* Color Selection Dialog                               */
/*------------------------------------------------------*/

void cancel_color(GtkWidget *widget, GtkWidget *color)
{
	if (color)
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(color), FALSE);		
	destroy_dialog(NULL, colordialog);
}

void do_color(GtkWidget *widget, GtkColorSelection *colorsel)
{
        gdouble color[3];
        GdkColor text_color;
	char *open_tag;

	open_tag = malloc(30);

        gtk_color_selection_get_color (colorsel, color);

        text_color.red = ((guint16)(color[0]*65535))>>8;
        text_color.green = ((guint16)(color[1]*65535))>>8;
        text_color.blue = ((guint16)(color[2]*65535))>>8;
	
	g_snprintf(open_tag, 23, "<FONT COLOR=\"#%02X%02X%02X\">", text_color.red, text_color.green, text_color.blue);

	surround(entry, open_tag, "</FONT>");
#ifdef DEBUG	
	printf("#%02X%02X%02X\n", text_color.red, text_color.green, text_color.blue);
#endif
	cancel_color(NULL, NULL);
}

void show_color_dialog(GtkWidget *entrye, GtkWidget *color)
{
        GtkWidget *colorsel;

        if (!colordialog) {
     
		entry = entrye; 

	        colordialog = gtk_color_selection_dialog_new("Select Text Color");
                colorsel = GTK_COLOR_SELECTION_DIALOG(colordialog)->colorsel;

                gtk_signal_connect(GTK_OBJECT(colordialog), "delete_event", GTK_SIGNAL_FUNC(cancel_color), color);

                gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colordialog)->ok_button), "clicked", GTK_SIGNAL_FUNC(do_color), colorsel);

                gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colordialog)->cancel_button), "clicked", GTK_SIGNAL_FUNC(cancel_color), color);
        }

        gtk_widget_show(colordialog);
        gdk_window_raise(colordialog->window);
}

/*------------------------------------------------------------------------*/
/*  The dialog for import/export                                          */
/*------------------------------------------------------------------------*/

void do_export(GtkWidget *w, void *dummy)
{
        FILE *f;
        char *buf = g_malloc(BUF_LONG);
        char *file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(exportdialog));

        if ((f = fopen(file,"w"))) {
                build_config(buf, BUF_LONG - 1);
                fprintf(f, "%s\n", buf);
                fclose(f);
                chmod(buf, S_IRUSR | S_IWUSR);
        } else {
                g_snprintf(buf, BUF_LONG / 2, "69:%s", file);
                show_error_dialog(buf);
        }
        destroy_dialog(NULL, exportdialog);
        exportdialog = NULL;
        
        g_free(buf);
        
        
}

	
void show_export_dialog()
{
        char *buf = g_malloc(BUF_LEN);
        if (!exportdialog) {
                exportdialog = gtk_file_selection_new("Gaim - Export Buddy List");

                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(exportdialog));

                g_snprintf(buf, BUF_LEN - 1, "%s/gaim.buddy", getenv("HOME"));
                
                gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdialog), buf);
                gtk_signal_connect(GTK_OBJECT(exportdialog), "destroy",
                                   GTK_SIGNAL_FUNC(destroy_dialog), exportdialog);
                
                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(exportdialog)->ok_button),
                                   "clicked", GTK_SIGNAL_FUNC(do_export), NULL);
                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(exportdialog)->cancel_button),
                                   "clicked", GTK_SIGNAL_FUNC(destroy_dialog), exportdialog);
                

        }

        gtk_widget_show(exportdialog);
        gdk_window_raise(exportdialog->window);

}

void do_import(GtkWidget *w, void *dummy)
{
        struct category *c = croot, *t;
        struct person *p, *n;
        char *buf = g_malloc(BUF_LONG);
        char *buf2;
        char *first = g_malloc(64);
        char *file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(importdialog));
        FILE *f;
        
        if (!(f = fopen(file,"r"))) {
                g_snprintf(buf, BUF_LONG / 2, "169:%s", file);
                show_error_dialog(buf);
                destroy_dialog(NULL, importdialog);
                importdialog = NULL;
                return;
        }
                
        fgets(first, 64, f);

        if (!strcasecmp(first, "Config {\n")) {
                printf("Windoze file!\n");
                destroy_dialog(NULL, importdialog);
                importdialog = NULL;
                return;
        } else if (buf[0] == 'm') {
                printf("TiK file!\n");
                buf2 = buf;
                buf = g_malloc(1025);
                g_snprintf(buf, 1024, "toc_set_config {%s}\n", buf2);
                g_free(buf2);
        }

                
        fseek(f, 0, SEEK_SET);

        fread(buf, BUF_LONG, 1, f);

        while(c) {
                p = c->members;
                t = c->next;
                while(p) {
                        n = p->next;
                        remove_person(c->name, p->name);
                        p = n;
                }
                remove_category(c->name);
                c = t;
        }
        
        parse_buddy_list(buf, 0);

        build_config(buf, BUF_LONG/2);
        sflap_send(buf, -1, TYPE_DATA);
        
        build_edit_tree();
        build_permit_tree();
        
        destroy_dialog(NULL, importdialog);
        importdialog = NULL;
        
        g_free(buf);
        
}

void show_import_dialog()
{
        char *buf = g_malloc(BUF_LEN);
        if (!importdialog) {
                importdialog = gtk_file_selection_new("Gaim - Import Buddy List");

                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(importdialog));

                g_snprintf(buf, BUF_LEN - 1, "%s/", getenv("HOME"));
                
                gtk_file_selection_set_filename(GTK_FILE_SELECTION(importdialog), buf);
                gtk_signal_connect(GTK_OBJECT(importdialog), "destroy",
                                   GTK_SIGNAL_FUNC(destroy_dialog), importdialog);
                
                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(importdialog)->ok_button),
                                   "clicked", GTK_SIGNAL_FUNC(do_import), NULL);
                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(importdialog)->cancel_button),
                                   "clicked", GTK_SIGNAL_FUNC(destroy_dialog), importdialog);
                

        }

        gtk_widget_show(importdialog);
        gdk_window_raise(importdialog->window);
}
