/*
 * gaim AOL Instant Messenger Client
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/signal.h>

#ifdef ESD_SOUND
#include <esd.h>
#endif // ESD_SOUND

#include "aim.h"
#include "BuddyArrive.h"
#include "BuddyLeave.h"
#include "Send.h"
#include "Receive.h"

#ifndef ESD_SOUND
static int pid=0;
#else


#endif // ESD_SOUND


#ifndef ESD_SOUND
void child_handler(int signal)
{
	waitpid(0, NULL,WNOHANG);
	pid = 0;
}

static void iot_handler(int signal) {
	exit(0);
}

static void play(char *data, int size)
{
	int fd;
	int status;
	
	signal(SIGCHLD,child_handler);
	
	if (pid > 0) {
		waitpid(pid, &status, WNOHANG);
		if (!WIFEXITED(status) && !WIFSIGNALED(status)) 
			return;
	}
	
	pid = fork();
	
	if (pid <0)
		return;
	else if (pid == 0) {
		int x;
		signal(SIGIOT, iot_handler);
		for (x=0;x<255;x++)
			close(x);
		fd = open("/dev/audio", O_WRONLY | O_NONBLOCK | O_EXCL);
		if (fd < 0)
			exit(0);
		write(fd, data, size);
		close(fd);
		exit(0);
	}
}
#else

/*
** This routine converts from ulaw to 16 bit linear.
**
** Craig Reese: IDA/Supercomputing Research Center
** 29 September 1989
**
** References:
** 1) CCITT Recommendation G.711  (very difficult to follow)
** 2) MIL-STD-188-113,"Interoperability and Performance Standards
**     for Analog-to_Digital Conversion Techniques,"
**     17 February 1987
**
** Input: 8 bit ulaw sample
** Output: signed 16 bit linear sample
** Z-note -- this is from libaudiofile.  Thanks guys!
*/

int _af_ulaw2linear (unsigned char ulawbyte)
{
  static int exp_lut[8] = {0,132,396,924,1980,4092,8316,16764};
  int sign, exponent, mantissa, sample;

  ulawbyte = ~ulawbyte;
  sign = (ulawbyte & 0x80);
  exponent = (ulawbyte >> 4) & 0x07;
  mantissa = ulawbyte & 0x0F;
  sample = exp_lut[exponent] + (mantissa << (exponent + 3));
  if (sign != 0) sample = -sample;

  return(sample);
}



static void play(unsigned char *data, int size)
{
        int fd, i;
	esd_format_t format = ESD_BITS16 | ESD_STREAM | ESD_PLAY | ESD_MONO;
        guint16 *lineardata = g_malloc(size * 2);
	
	
	fd = esd_play_stream(format, 8012, NULL, "gaim");

	for (i=0; i<size; i++)
		lineardata[i] = _af_ulaw2linear(data[i]);
	
	write(fd, lineardata, size * 2);

	close(fd);
	g_free(lineardata);


}
#endif // ESD_SOUND




void play_sound(int sound)
{

	switch(sound) {
	case BUDDY_ARRIVE:
		if (sounds)
			play(BuddyArrive, sizeof(BuddyArrive));\
		break;
	case BUDDY_LEAVE:
		if (sounds)
			play(BuddyLeave, sizeof(BuddyLeave));\
		break;
	case SEND:
		if (extrasounds)
			play(Send, sizeof(Send));\
		break;
	case RECEIVE:
		if (extrasounds)
			play(Receive, sizeof(Receive));\
		break;
        }

			
}
