/*
 * gaim AOL Instant Messenger Client
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <netdb.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/socket.h>
#include "aim.h"

/* descriptor for talking to TOC */
static int toc_fd;
static int seqno;
static unsigned int peer_ver=0;
static int state;
static int inpa=-1;
static guint idle_timer;
static time_t lastsent;
static int is_idle;

struct conversation *cnvroot;
struct buddy_chat *chatroot=NULL;

char toc_addy[16];
char *quad_addr;

GList *permit;
GList *deny;
int permdeny;

char ourname[80];
struct away_message *awaymessage;

unsigned int *get_address(char *hostname)
{
	struct hostent *hp;
	unsigned int *sin=NULL;
	if ((hp=gethostbyname(hostname))) {
		sin = (unsigned int *)g_new(struct sockaddr_in, 1);
		memcpy(sin, hp->h_addr, hp->h_length);
	}
	return sin;
}

char *condense(char *s)
{
	static char buf[BUF_LEN];
	int x=0;
	while(*s) {
		if (*s != ' ') {
			buf[x] = *s;
			x++;
		}
		s++;
	}
	buf[x]='\0';
	return buf;
}

int connect_address(unsigned int addy, unsigned short port)
{
	struct sockaddr_in sin;

	sin.sin_addr.s_addr = addy;
	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	
	toc_fd = socket(AF_INET, SOCK_STREAM, 0);
	
	if (toc_fd > -1) {
		quad_addr=strdup(inet_ntoa(sin.sin_addr));
		if (connect(toc_fd, &sin, sizeof(sin)) > -1) {
			return 0;
		}
	}
	return -1;
}

void toc_close()
{
	if (inpa > 0)
		gdk_input_remove(inpa);
	close(toc_fd);
	toc_fd=-1;
	inpa=-1;
}

unsigned char *roast_password(char *pass)
{
	/* Trivial "encryption" */
	static char rp[256];
	static char *roast = ROAST;
	int pos=2;
	int x;
	strcpy(rp, "0x");
	for (x=0;(x<150) && pass[x]; x++) 
		pos+=sprintf(&rp[pos],"%02x", pass[x] ^ roast[x % strlen(roast)]);
	rp[pos]='\0';
	return rp;
}

int escape_message(char *msg)
{
	char *c, *cpy;
	int cnt=0;
	/* Assumes you have a buffer able to cary at least BUF_LEN * 2 bytes */
	if (strlen(msg) > BUF_LEN) {
		fprintf(stderr, "Warning:  truncating message to 1024 bytes\n");
		msg[1023]='\0';
	}
	
	cpy = strdup(msg);
	c = cpy;
	while(*c) {
		switch(*c) {
		case '$':
		case '[':
		case ']':
		case '(':
		case ')':
		case '#':
			msg[cnt++]='\\';
			/* Fall through */
		default:
			msg[cnt++]=*c;
		}
		c++;
	}
	msg[cnt]='\0';
	g_free(cpy);
	return cnt;
}

int escape_text(char *msg)
{
	char *c, *cpy;
	int cnt=0;
	/* Assumes you have a buffer able to cary at least BUF_LEN * 2 bytes */
	if (strlen(msg) > BUF_LEN) {
		fprintf(stderr, "Warning:  truncating message to 1024 bytes\n");
		msg[1023]='\0';
	}
	
	cpy = strdup(msg);
	c = cpy;
	while(*c) {
		switch(*c) {
		case '{':
		case '}':
		case '\\':
		case '"':
			msg[cnt++]='\\';
			/* Fall through */
		default:
			msg[cnt++]=*c;
		}
		c++;
	}
	msg[cnt]='\0';
	g_free(cpy);
	return cnt;
}

char *print_header(void *hdr_v)
{
	static char s[80];
	struct sflap_hdr *hdr = (struct sflap_hdr *)hdr_v;
	g_snprintf(s,sizeof(s), "[ ast: %c, type: %d, seqno: %d, len: %d ]",
		hdr->ast, hdr->type, ntohs(hdr->seqno), ntohs(hdr->len));
	return s;
}

void print_buffer(char *buf, int len)
{
#if 0
	int x;
	printf("[ ");
	for (x=0;x<len;x++) 
		printf("%d ", buf[x]);
	printf("]\n");
	printf("[ ");
	for (x=0;x<len;x++)
		printf("%c ", buf[x]);
	printf("]\n");
#endif
}

int sflap_send(char *buf, int olen, int type)
{
	int len;
	int slen=0;
	struct sflap_hdr hdr;
	char obuf[1024];

       /* Are we idle?  If so, not anymore */
	if (is_idle > 0) {
		char ibuf[BUF_LONG];
		is_idle = 0;
		g_snprintf(ibuf, sizeof(ibuf), "toc_set_idle 0");
		sflap_send(ibuf, -1, TYPE_DATA);
	} else if (!is_idle)
		time(&lastsent);

	
	if (olen < 0)
		len = escape_message(buf);
	else
		len = olen;
	hdr.ast = '*';
	hdr.type = type;
	hdr.seqno = htons(seqno++ & 0xffff);
	hdr.len = htons(len + (type == TYPE_SIGNON ? 0 : 1));
/*	fprintf(stdout,"Escaped message is '%s'\n",buf); */
	memcpy(obuf, &hdr, sizeof(hdr));
	slen += sizeof(hdr);
	memcpy(&obuf[slen], buf, len);
	slen += len;
	if (type != TYPE_SIGNON) {
		obuf[slen]='\0';
		slen += 1;
	}
	print_buffer(obuf, slen);

	return write(toc_fd, obuf, slen);
}

int wait_reply(char *buffer, int buflen)
{
	int res;
	struct sflap_hdr *hdr=(struct sflap_hdr *)buffer;
	char *c;
	res = read(toc_fd, buffer, sizeof(struct sflap_hdr));
        if (res < 0)
		return res;
	res += read(toc_fd, buffer + sizeof(struct sflap_hdr), ntohs(hdr->len));
	if (res >= sizeof(struct sflap_hdr)) 
		buffer[res]='\0';
	else
		return res - sizeof(struct sflap_hdr);
		
#if 0
	fprintf(stdout, "Rcv: %s %s\n",print_header(buffer), "");
#endif
	switch(hdr->type) {
	case TYPE_SIGNON:
		memcpy(&peer_ver, buffer + sizeof(struct sflap_hdr), 4);
		peer_ver = ntohl(peer_ver);
		seqno = ntohs(hdr->seqno);
		state = STATE_SIGNON_REQUEST;
		break;
	case TYPE_DATA:
		if (!strncasecmp(buffer + sizeof(struct sflap_hdr), "SIGN_ON:", strlen("SIGN_ON:")))
			state = STATE_SIGNON_ACK;
		else if (!strncasecmp(buffer + sizeof(struct sflap_hdr), "CONFIG:", strlen("CONFIG:"))) {
			state = STATE_CONFIG;
		} else if (state != STATE_CONFIG && !strncasecmp(buffer + sizeof(struct sflap_hdr), "ERROR:", strlen("ERROR:"))) {
			c = strtok(buffer + sizeof(struct sflap_hdr) + strlen("ERROR:"), ":");
			show_error_dialog(c);
		}
#ifdef DEBUG
		fprintf(stderr, "Data: %s\n",buffer + sizeof(struct sflap_hdr));
#endif
		break;
	default:
		fprintf(stderr, "Unknown/unimplemented packet type %d\n",hdr->type);
	}
        return res;
}




static
void close_warned(GtkWidget *w, GtkWidget *w2)
{
        gtk_widget_destroy(w2);
}

static
void close_invite(GtkWidget *w, GtkWidget *w2)
{
	gtk_widget_destroy(w2);
}

static
void chat_invite_callback(GtkWidget *w, GtkWidget *w2)
{
	int i = (int)gtk_object_get_user_data(GTK_OBJECT(w2));
	char *buf = g_malloc(256);
	g_snprintf(buf, 255, "toc_chat_accept %d",  i);
        sflap_send(buf, -1, TYPE_DATA);
	gtk_widget_destroy(w2);
}



void toc_callback( gpointer          data,
                            gint              source, 
                            GdkInputCondition condition )
{
	char buf[BUF_LEN];
	char *c;
	char *l;
	
        if (wait_reply(buf, sizeof(buf)) < 0) {
//                printf("Hello\n");
		signoff();
		gtk_timeout_remove(idle_timer);
		hide_progress("Connection Closed");
		return;
	};
	c=strtok(buf+sizeof(struct sflap_hdr),":");	/* Ditch the first part */
	if (!strcasecmp(c,"UPDATE_BUDDY")) {
		char *uc;
		
	
		struct person *p = NULL;
		
		c = strtok(NULL,":"); /* c is name */

		p = get_person(c);
		
		if (!p) {
			printf("Error, no such person\n");
			return;
                }
//                printf("Person %s found\n", p->name);

                /* This code will 'align' the name from the TOC */
                /* server with what's in our record.  We want to */
                /* store things how THEY want it... */
                if (strcmp(c, p->name)) {

                        struct conversation *cnv = cnvroot;
                        char *who = g_malloc(80);
                        char *tmp;
                        char *buf2 = g_malloc(1025);
#ifdef DEBUG
                        printf("'%s' | '%s' compare\n", c, p->name);
#endif

                        
                        strcpy(who, condense(c));

                        while(cnv) {
                                if (!strcasecmp(who, condense(cnv->name))) {
                                        g_snprintf(cnv->name, sizeof(cnv->name), "%s", c);
                                        if (is_logging(c))
                                                g_snprintf(who, 63, CONVERSATION_TITLE, c);
                                        else
                                                g_snprintf(who, 63, LOG_CONVERSATION_TITLE, c);
                                        tmp = GTK_WINDOW(cnv->window)->title;
                                        gtk_window_set_title(GTK_WINDOW(cnv->window), who);
                                        g_free(tmp);
                                        // no free 'who', set_title needs it.
                                        // FIXME should free the old one....
                                        break;
                                }
                                cnv = cnv->next;
                        }
                        
                        g_snprintf(p->name, sizeof(p->name), "%s", c);
                        gtk_label_set_text(GTK_LABEL(p->label), p->name);

                        // okay lets save the new config... */
                        build_config(buf2, 1024);
                        sflap_send(buf2, -1, TYPE_DATA);
                        g_free(buf2);
                }
                        
                
		/* Parse! */

                
		
		l = strtok(NULL,":"); /* l is T/F logged status */


                if (!strncasecmp(l,"T",1)) {
                        if (!p->present) {
                                p->present = 1;
                                do_pounce(p->name);
                        }
                } else
			p->present = 0;
		
		sscanf(strtok(NULL, ":"), "%d", &p->evil);
		
		sscanf(strtok(NULL, ":"), "%ld", &p->signon);
		
		sscanf(strtok(NULL, ":"), "%d", &p->idle);
		
		uc = strtok(NULL, ":");
/*		printf("%s:%s\n", c, uc); */
		p->uc = 0; /* Initialize this dumbass! *slap forehead */
		
		
		
		if (uc[0] == 'A')
			p->uc |= UC_AOL;
		
		switch(uc[1]) {
			case 'A':
				p->uc |= UC_ADMIN;
				break;
			case 'U':
				p->uc |= UC_UNCONFIRMED;
				break;
			case 'O':
				p->uc |= UC_NORMAL;
				break;
			default:
				break;
		}
		
		set_buddy(p);
		
	} else 
	if (!strcasecmp(c, "ERROR")) {
		c = strtok(NULL,":");
		show_error_dialog(c);
	} else
	if (!strcasecmp(c, "NICK")) {
		c = strtok(NULL,":");
		g_snprintf(ourname, sizeof(ourname), "%s", c);
#ifdef DEBUG
		fprintf(stdout, "We are %s\n",ourname);
#endif
	} else
	if (!strcasecmp(c, "IM_IN")) {
		char *away, *message;
                struct conversation *cnv;
                
		c = strtok(NULL,":");
		away = strtok(NULL,":");
		message = away;
		while(*message && (*message != ':')) message++;
                message++;

                cnv = find_conversation(c);

                if (cnv == NULL)
                        cnv = new_conversation(c);
                
                conv_write(cnv, message, away);
		if (awaymessage != NULL) {
			time_t t;
			char tmp[BUF_LONG];

                        time(&t);

                        printf("Here?\n");

			if ((cnv == NULL) || (t - cnv->sent_away) < 120)
				return;

                        printf("Here2\n");
                        
			cnv->sent_away = t;

			if (is_idle)
				is_idle = -1;
                    
			g_snprintf(tmp, 1024, "toc_send_im %s \"%s\" auto", c, awaymessage->message);
			sflap_send(tmp, -1, TYPE_DATA);

		        if (is_idle == -1)
                                is_idle = 1;

                        write_to_conv(cnv->text, ourname, awaymessage->message, WFLAG_SEND | WFLAG_AUTO);
			
		}


	} else if (!strcasecmp(c, "GOTO_URL")) {
		char *name;
		char *url;

		char tmp[256];
		
		name = strtok(NULL, ":");
		url = strtok(NULL, ":");


		g_snprintf(tmp, sizeof(tmp), "http://%s:%d/%s", toc_addy, TOC_PORT, url);
#ifdef USE_GHTTP
		new_show_html(NULL,fix_url(tmp));
#else
		show_html(NULL, fix_url(tmp));
#endif /* USE_GHTTP */
//		fprintf(stdout, "Name: %s\n%s\n", name, url);
		
        } else if (!strcasecmp(c, "EVILED")) {
                int lev;
                char *name = NULL;
                char *buf2 = g_malloc(1024);
                GtkWidget *d, *label, *close;
                
                sscanf(strtok(NULL, ":"), "%d", &lev);
                name = strtok(NULL, ":");

                printf("%s | %d\n", name, lev);

                
                g_snprintf(buf2, 1023, "You have just been warned by %s.\nYour new warning level is %d./%%",
                           ((name == NULL) ? "an anonymous person" : name) , lev);
                

                d = gtk_dialog_new();

                label = gtk_label_new(buf2);
                gtk_widget_show(label);
                close = gtk_button_new_with_label("Close");
                gtk_widget_show(close);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),
                                   label, FALSE, FALSE, 5);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->action_area),
                                   close, FALSE, FALSE, 5);
		
                gtk_window_set_title(GTK_WINDOW(d), "Warned");
                gtk_signal_connect(GTK_OBJECT(close), "clicked", GTK_SIGNAL_FUNC(close_warned), d);
                gtk_widget_show(d);
        } else if (!strcasecmp(c, "CHAT_JOIN")) {
		char *name;
		struct buddy_chat *b;
		b = chatroot;
	
		if (!b) {
			b = (struct buddy_chat *)g_new(struct buddy_chat, 1);
			chatroot = b;
		} else {
			while(b->next)
				b = b->next;
			b->next = (struct buddy_chat *)g_new(struct buddy_chat, 1);
			b = b->next;
		}
		b->next = NULL;
		sscanf(strtok(NULL, ":"), "%d", &b->id);
		name = strtok(NULL, ":");
		g_snprintf(b->name, 80, "%s", name);
		show_new_buddy_chat(b);

	} else
	if (!strcasecmp(c, "CHAT_UPDATE_BUDDY")) {
		int id;
		char *in;
		char *buddy;

		struct buddy_chat *b = chatroot;
		

		sscanf(strtok(NULL, ":"), "%d", &id);

		in = strtok(NULL, ":");

		while(b) {
			if (id == b->id)
				break;	
			b = b->next;

		}
		if (!b)
			return;

		
		if (!strcasecmp(in, "T")) {
			while((buddy = strtok(NULL, ":")) != NULL) {
				add_chat_buddy(b, buddy);
			}
		} else {
			while((buddy = strtok(NULL, ":")) != NULL) {
				remove_chat_buddy(b, buddy);
			}
		}

	} else
	if (!strcasecmp(c, "CHAT_LEFT")) {
		int id;
		struct buddy_chat *b = chatroot;
		struct buddy_chat *p = NULL;


		sscanf(strtok(NULL, ":"), "%d", &id);
		while(b) {
			if (id == b->id) {
				break;	
			}
			p = (struct buddy_chat *)b;
			b = b->next;

		}
		if (!b)
			return;

		gtk_widget_destroy(GTK_WIDGET(b->window));
		
		if (p) {
			p->next = b->next;
		} else {
			chatroot = NULL;
		}
			
		
		g_free((void *)b);
		b = NULL;

	} else
	if (!strcasecmp(c, "CHAT_IN")) {

		int id, w;
		char *m;
            

		char *who, *whisper;
		struct buddy_chat *b = chatroot;

	
		sscanf(strtok(NULL, ":"), "%d", &id);
		who = strtok(NULL, ":");
		whisper = strtok(NULL, ":");
		m = whisper;
                while(*m && (*m != ':')) m++;
                m++;
                
		while(b) {
			if (id == b->id)
				break;	
			b = b->next;

		}
		if (!b)
			return;
		if (!strcasecmp(whisper, "T"))
			w = 1;
		else	
			w = 0;
	
		chat_write(b, who, w, m);

	} else
	if (!strcasecmp(c, "CHAT_INVITE")) {
		GtkWidget *d;
		GtkWidget *label;
		GtkWidget *yesbtn;
		GtkWidget *nobtn;
		char *name;
		char *who;
		char *message;
		int id;

		char buf2[BUF_LONG];

		name = strtok(NULL, ":");
		sscanf(strtok(NULL, ":"), "%d", &id);
		who = strtok(NULL, ":");
                message = strtok(NULL, ":");

		g_snprintf(buf2, sizeof(buf2), "User '%s' invites you to buddy chat room: '%s'\n%s", who, name, message);

		d = gtk_dialog_new();


		label = gtk_label_new(buf2);
		gtk_widget_show(label);
		yesbtn = gtk_button_new_with_label("Yes");
		gtk_widget_show(yesbtn);
		nobtn = gtk_button_new_with_label("No");
		gtk_widget_show(nobtn);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),
			label, FALSE, FALSE, 5);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->action_area), 
			yesbtn, FALSE, FALSE, 5);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->action_area),
			nobtn, FALSE, FALSE, 5);


//		gtk_widget_set_usize(d, 200, 110);
		gtk_object_set_user_data(GTK_OBJECT(d), (void *)id);


		gtk_window_set_title(GTK_WINDOW(d), "Buddy chat invite");
		gtk_signal_connect(GTK_OBJECT(nobtn), "clicked", GTK_SIGNAL_FUNC(close_invite), d);
		gtk_signal_connect(GTK_OBJECT(yesbtn), "clicked", GTK_SIGNAL_FUNC(chat_invite_callback), d);

	
		gtk_widget_show(d);
	} else {
		fprintf(stdout,"don't know what to do with %s\n", c);
	}
	
}


int toc_signon(char *username, char *password)
{
	char buf[BUF_LONG];
	int res;
	struct signon so;
	
	strncpy(ourname, username, sizeof(ourname));
	
	if ((res = write(toc_fd, FLAPON, strlen(FLAPON))) < 0)
		return res;
	/* Wait for signon packet */

	state = STATE_FLAPON;

	if ((res = wait_reply(buf, sizeof(buf)) < 0))
		return res;
	
	if (state != STATE_SIGNON_REQUEST) {
#ifdef DEBUG
		fprintf(stderr, "State should be %d, but is %d instead\n", STATE_SIGNON_REQUEST, state);
#endif
		return -1;
	}
	
	/* Compose a response */
	
	g_snprintf(so.username, sizeof(so.username), "%s", username);
	so.ver = ntohl(1);
	so.tag = ntohs(1);
	so.namelen = htons(strlen(so.username));	
	
	sflap_send((char *)&so, ntohs(so.namelen) + 8, TYPE_SIGNON);
	
	g_snprintf(buf, sizeof(buf), 
		"toc_signon %s %d %s %s %s \"%s\"",
		AUTH_HOST, AUTH_PORT, condense(username), roast_password(password), LANGUAGE, REVISION);
		
	return sflap_send(buf, -1, TYPE_DATA);
}

int toc_wait_signon()
{
	/* Wait for the SIGNON to be approved */
	char buf[BUF_LEN];
	int res;
	res = wait_reply(buf, sizeof(buf));
	if (res < 0)
		return res;
	if (state != STATE_SIGNON_ACK) {
#ifdef DEBUG
		fprintf(stderr, "State should be %d, but is %d instead\n",STATE_SIGNON_ACK, state);
#endif
		return -1;
	}
	return 0;
}

static gint check_idle()
{
	char buf[BUF_LONG];
	time_t t;

	if (is_idle)
		return TRUE;

	time(&t);


	if ((t - lastsent) > 600) { /* 15 minutes! */
		is_idle = -1;
		g_snprintf(buf, sizeof(buf), "toc_set_idle %d", (int)(t - lastsent));
		sflap_send(buf, -1, TYPE_DATA);
		is_idle = 1;
	}
	return TRUE;

}


char *toc_wait_config()
{
	/* Waits for configuration packet, returning the contents of the packet */
	static char buf[BUF_LEN];
	int res;
        res = wait_reply(buf, sizeof(buf));
	if (res < 0)
		return NULL;
	if (state != STATE_CONFIG) {
#ifdef DEBUG
		fprintf(stderr, "State should be %d, but is %d instead\n",STATE_CONFIG, state);
#endif
		return NULL;
	}
	/* At this point, it's time to setup automatic handling of incoming packets */
	inpa = gdk_input_add(toc_fd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, toc_callback, NULL);
        idle_timer = gtk_timeout_add(100000, (GtkFunction)check_idle, NULL);
	return buf;
}
